import os
import unittest
from unittest.mock import patch


class TestMcpInputPolicy(unittest.TestCase):
    def test_mcp_transport_rejects_path_source(self) -> None:
        from gravtice.genai import GenAIError
        from gravtice.genai.client import Client
        from gravtice.genai.types import (
            GenerateRequest,
            Message,
            OutputSpec,
            Part,
            PartSourcePath,
        )

        with patch.dict(os.environ, {"GENAI_CALLING_TRANSPORT": "sse"}, clear=False):
            client = Client()
            req = GenerateRequest(
                model="unknown:demo",
                input=[
                    Message(
                        role="user",
                        content=[
                            Part(
                                type="image",
                                mime_type="image/png",
                                source=PartSourcePath(path="/tmp/x.png"),
                            )
                        ],
                    )
                ],
                output=OutputSpec(modalities=["text"]),
            )
            with self.assertRaises(GenAIError) as cm:
                client.generate(req)
            self.assertEqual(cm.exception.info.type, "InvalidRequestError")
            self.assertIn("MCP transport does not support", cm.exception.info.message)

    def test_mcp_transport_rejects_bytes_source(self) -> None:
        from gravtice.genai import GenAIError
        from gravtice.genai.client import Client
        from gravtice.genai.types import (
            GenerateRequest,
            Message,
            OutputSpec,
            Part,
            PartSourceBytes,
        )

        with patch.dict(os.environ, {"GENAI_CALLING_TRANSPORT": "sse"}, clear=False):
            client = Client()
            req = GenerateRequest(
                model="unknown:demo",
                input=[
                    Message(
                        role="user",
                        content=[
                            Part(
                                type="audio",
                                mime_type="audio/wav",
                                source=PartSourceBytes(data=b"123"),
                            )
                        ],
                    )
                ],
                output=OutputSpec(modalities=["text"]),
            )
            with self.assertRaises(GenAIError) as cm:
                client.generate(req)
            self.assertEqual(cm.exception.info.type, "InvalidRequestError")
            self.assertIn("MCP transport does not support", cm.exception.info.message)

    def test_mcp_transport_allows_base64_source(self) -> None:
        from gravtice.genai import GenAIError
        from gravtice.genai.client import Client
        from gravtice.genai.types import (
            GenerateRequest,
            Message,
            OutputSpec,
            Part,
            PartSourceBytes,
        )

        with patch.dict(os.environ, {"GENAI_CALLING_TRANSPORT": "sse"}, clear=False):
            client = Client()
            req = GenerateRequest(
                model="unknown:demo",
                input=[
                    Message(
                        role="user",
                        content=[
                            Part(
                                type="image",
                                mime_type="image/png",
                                source=PartSourceBytes(data="AA==", encoding="base64"),
                            )
                        ],
                    )
                ],
                output=OutputSpec(modalities=["text"]),
            )
            with self.assertRaises(GenAIError) as cm:
                client.generate(req)
            self.assertEqual(cm.exception.info.type, "InvalidRequestError")
            self.assertIn("unknown provider", cm.exception.info.message)



import unittest
from unittest.mock import patch


class TestModesInference(unittest.TestCase):
    def test_modes_for_common_openai_models(self) -> None:
        from gravtice.genai.reference import get_sdk_supported_models_for_provider

        rows = get_sdk_supported_models_for_provider("openai")
        by_model = {r["model"]: r for r in rows}

        self.assertEqual(
            by_model["openai:gpt-4o-mini"]["modes"], ["sync", "stream", "async"]
        )
        self.assertEqual(by_model["openai:dall-e-3"]["modes"], ["sync", "async"])
        self.assertEqual(by_model["openai:sora-2"]["modes"], ["sync", "job", "async"])

    def test_overrides_can_force_job_and_change_order(self) -> None:
        from gravtice.genai.reference import get_sdk_supported_models_for_provider

        overrides = {"openai": {"gpt-4o-mini": {"supports_job": True}}}
        with patch("gravtice.genai.reference.catalog.CAPABILITY_OVERRIDES", overrides):
            rows = get_sdk_supported_models_for_provider("openai")

        by_model = {r["model"]: r for r in rows}
        self.assertEqual(
            by_model["openai:gpt-4o-mini"]["modes"], ["sync", "stream", "job", "async"]
        )
        self.assertIn(
            "可能返回 running(job)", " ".join(by_model["openai:gpt-4o-mini"]["notes"])
        )

    def test_overrides_can_disable_stream(self) -> None:
        from gravtice.genai.reference import get_sdk_supported_models_for_provider

        overrides = {"openai": {"gpt-4o-mini": {"supports_stream": False}}}
        with patch("gravtice.genai.reference.catalog.CAPABILITY_OVERRIDES", overrides):
            rows = get_sdk_supported_models_for_provider("openai")

        by_model = {r["model"]: r for r in rows}
        self.assertEqual(by_model["openai:gpt-4o-mini"]["modes"], ["sync", "async"])



from __future__ import annotations

from .catalog import (
    get_model_catalog,
    get_sdk_supported_models,
    get_sdk_supported_models_for_provider,
    get_supported_providers,
)
from .mappings import get_parameter_mappings

__all__ = [
    "get_model_catalog",
    "get_parameter_mappings",
    "get_sdk_supported_models",
    "get_sdk_supported_models_for_provider",
    "get_supported_providers",
]



import unittest
from dataclasses import dataclass
from typing_extensions import TypedDict


class TestOutputParserTool(unittest.TestCase):
    def test_build_output_parser_tool_accepts_python_types(self) -> None:
        from gravtice.genai._internal.errors import GenAIError
        from gravtice.genai.tools.output_parser import build_output_parser_tool

        tool = build_output_parser_tool(
            {"type": "object", "properties": {"a": {"type": "integer"}}}
        )
        self.assertIsInstance(tool.parameters, dict)

        try:
            from pydantic import BaseModel
        except ModuleNotFoundError:
            with self.assertRaises(GenAIError) as ctx:
                build_output_parser_tool(list[int])
            self.assertEqual(ctx.exception.info.type, "NotSupportedError")
            return

        class Model(BaseModel):
            a: int

        class TD(TypedDict):
            a: int

        @dataclass
        class DC:
            a: int

        cases = [
            ("pydantic.BaseModel", Model),
            ("TypedDict", TD),
            ("dataclass", DC),
            ("typing", list[int]),
        ]

        for name, schema_type in cases:
            with self.subTest(name=name):
                tool = build_output_parser_tool(schema_type)
                self.assertIsInstance(tool.parameters, dict)
                assert isinstance(tool.parameters, dict)
                props = tool.parameters.get("properties")
                self.assertIsInstance(props, dict)
                assert isinstance(props, dict)
                out_schema = props.get("output")
                self.assertIsInstance(out_schema, dict)

    def test_extract_output_from_response(self) -> None:
        from gravtice.genai._internal.errors import GenAIError
        from gravtice.genai.tools.output_parser import extract_output_from_response
        from gravtice.genai.types import GenerateResponse, Message, Part

        resp = GenerateResponse(
            id="r1",
            provider="dummy",
            model="dummy:demo",
            status="completed",
            output=[
                Message(
                    role="assistant",
                    content=[
                        Part.tool_call(
                            name="genai_output_parser",
                            arguments={"output": {"a": 1}},
                        )
                    ],
                )
            ],
        )
        self.assertEqual(extract_output_from_response(resp), {"a": 1})

        resp2 = GenerateResponse(
            id="r2",
            provider="dummy",
            model="dummy:demo",
            status="completed",
            output=[
                Message(role="assistant", content=[Part.from_text("no tool call")])
            ],
        )
        with self.assertRaises(GenAIError):
            extract_output_from_response(resp2)

    def test_parse_output(self) -> None:
        from gravtice.genai._internal.errors import GenAIError
        from gravtice.genai.tools.output_parser import parse_output
        from gravtice.genai.types import (
            Capability,
            GenerateRequest,
            GenerateResponse,
            Message,
            Part,
        )

        class DummyClient:
            def __init__(self, *, supports_tools: bool) -> None:
                self._supports_tools = supports_tools
                self.last_request: GenerateRequest | None = None

            def capabilities(self, model: str) -> Capability:
                return Capability(
                    input_modalities={"text"},
                    output_modalities={"text"},
                    supports_stream=False,
                    supports_job=False,
                    supports_tools=self._supports_tools,
                    supports_json_schema=False,
                )

            def generate(self, request: GenerateRequest):
                self.last_request = request
                return GenerateResponse(
                    id="r3",
                    provider="dummy",
                    model=request.model,
                    status="completed",
                    output=[
                        Message(
                            role="assistant",
                            content=[
                                Part.tool_call(
                                    name="genai_output_parser",
                                    arguments={"output": {"a": 1}},
                                )
                            ],
                        )
                    ],
                )

        schema = {
            "type": "object",
            "properties": {"a": {"type": "integer"}},
            "required": ["a"],
        }
        c = DummyClient(supports_tools=True)
        out = parse_output(c, model="dummy:demo", text="hello", json_schema=schema)
        self.assertEqual(out, {"a": 1})
        self.assertIsNotNone(c.last_request)
        assert c.last_request is not None
        self.assertEqual(c.last_request.tool_choice.mode, "tool")
        self.assertEqual(c.last_request.tool_choice.name, "genai_output_parser")
        self.assertTrue(
            c.last_request.input[0].content[0].require_text().endswith("hello")
        )

        with self.assertRaises(GenAIError) as ctx:
            parse_output(c, model="dummy:demo", text="   ", json_schema=schema)
        self.assertEqual(ctx.exception.info.type, "InvalidRequestError")

        c2 = DummyClient(supports_tools=False)
        with self.assertRaises(GenAIError) as ctx2:
            parse_output(c2, model="dummy:demo", text="hello", json_schema=schema)
        self.assertEqual(ctx2.exception.info.type, "NotSupportedError")



import unittest
from dataclasses import dataclass
from unittest.mock import patch
from typing_extensions import TypedDict


class TestOutputTextJsonSchema(unittest.TestCase):
    def test_client_coerces_python_type_json_schema(self) -> None:
        try:
            from pydantic import BaseModel
        except ModuleNotFoundError:
            self.skipTest("missing dependency: pydantic")

        from gravtice.genai.client import Client
        from gravtice.genai.types import (
            Capability,
            GenerateRequest,
            GenerateResponse,
            Message,
            OutputSpec,
            OutputTextSpec,
            Part,
        )

        class Model(BaseModel):
            a: int

        class TD(TypedDict):
            a: int

        @dataclass
        class DC:
            a: int

        class DummyAdapter:
            def __init__(self) -> None:
                self.last_schema = None

            def capabilities(self, model_id: str) -> Capability:  # noqa: ARG002
                return Capability(
                    input_modalities={"text"},
                    output_modalities={"text"},
                    supports_stream=False,
                    supports_job=False,
                )

            def generate(self, request: GenerateRequest, *, stream: bool):  # noqa: ARG002
                spec = request.output.text
                if spec is None:
                    raise AssertionError("expected output.text to be set")
                self.last_schema = spec.json_schema
                if not isinstance(self.last_schema, dict):
                    raise AssertionError("expected json_schema to be coerced to dict")
                return GenerateResponse(
                    id="r1",
                    provider="dummy",
                    model=request.model,
                    status="completed",
                    output=[Message(role="assistant", content=[Part.from_text("ok")])],
                )

        client = Client()
        adapter = DummyAdapter()

        cases = [
            ("pydantic.BaseModel", Model),
            ("TypedDict", TD),
            ("dataclass", DC),
            ("typing", list[int]),
        ]
        for name, schema_type in cases:
            with self.subTest(name=name):
                req = GenerateRequest(
                    model="dummy:demo",
                    input=[Message(role="user", content=[Part.from_text("hi")])],
                    output=OutputSpec(
                        modalities=["text"],
                        text=OutputTextSpec(format="json", json_schema=schema_type),
                    ),
                )
                with patch.object(client, "_adapter", return_value=adapter):
                    client.generate(req)
                assert isinstance(adapter.last_schema, dict)

    def test_gemini_converts_json_schema_dict(self) -> None:
        try:
            from pydantic import BaseModel
        except ModuleNotFoundError:
            self.skipTest("missing dependency: pydantic")

        from gravtice.genai.providers.gemini import GeminiAdapter
        from gravtice.genai.types import (
            GenerateRequest,
            Message,
            OutputSpec,
            OutputTextSpec,
            Part,
        )

        class Child(BaseModel):
            x: int

        class Parent(BaseModel):
            child: Child
            b: str | None = None

        schema = Parent.model_json_schema()
        req = GenerateRequest(
            model="google:gemini-1.5-flash",
            input=[Message(role="user", content=[Part.from_text("hi")])],
            output=OutputSpec(
                modalities=["text"],
                text=OutputTextSpec(format="json", json_schema=schema),
            ),
        )

        adapter = GeminiAdapter(api_key="dummy")
        body = adapter._generate_body(req, model_name="gemini-1.5-flash")
        gen_cfg = body.get("generationConfig")
        if not isinstance(gen_cfg, dict):
            raise AssertionError("missing generationConfig")
        resp_schema = gen_cfg.get("responseSchema")
        if not isinstance(resp_schema, dict):
            raise AssertionError("missing responseSchema")

        self.assertEqual(resp_schema.get("type"), "OBJECT")
        props = resp_schema.get("properties")
        self.assertIsInstance(props, dict)
        assert isinstance(props, dict)
        self.assertIn("child", props)
        self.assertIn("b", props)

        child_schema = props["child"]
        self.assertIsInstance(child_schema, dict)
        assert isinstance(child_schema, dict)
        self.assertEqual(child_schema.get("type"), "OBJECT")
        self.assertIsInstance(child_schema.get("properties"), dict)

        b_schema = props["b"]
        self.assertIsInstance(b_schema, dict)
        assert isinstance(b_schema, dict)
        self.assertEqual(b_schema.get("type"), "STRING")
        self.assertEqual(b_schema.get("nullable"), True)

    def test_rejects_gemini_schema_dict_in_json_schema(self) -> None:
        from gravtice.genai._internal.errors import GenAIError
        from gravtice.genai.client import Client
        from gravtice.genai.providers.gemini import GeminiAdapter
        from gravtice.genai.types import (
            Capability,
            GenerateRequest,
            Message,
            OutputSpec,
            OutputTextSpec,
            Part,
        )

        gemini_schema = {"type": "OBJECT", "properties": {"a": {"type": "STRING"}}}

        req = GenerateRequest(
            model="dummy:demo",
            input=[Message(role="user", content=[Part.from_text("hi")])],
            output=OutputSpec(
                modalities=["text"],
                text=OutputTextSpec(format="json", json_schema=gemini_schema),
            ),
        )

        class DummyAdapter:
            def capabilities(self, model_id: str) -> Capability:  # noqa: ARG002
                return Capability(
                    input_modalities={"text"},
                    output_modalities={"text"},
                    supports_stream=False,
                    supports_job=False,
                )

            def generate(self, request: GenerateRequest, *, stream: bool):  # noqa: ARG002
                raise AssertionError("should fail before adapter.generate()")

        client = Client()
        with patch.object(client, "_adapter", return_value=DummyAdapter()):
            with self.assertRaises(GenAIError) as ctx:
                client.generate(req)
            self.assertEqual(ctx.exception.info.type, "InvalidRequestError")

        req_google = GenerateRequest(
            model="google:gemini-1.5-flash",
            input=[Message(role="user", content=[Part.from_text("hi")])],
            output=OutputSpec(
                modalities=["text"],
                text=OutputTextSpec(format="json", json_schema=gemini_schema),
            ),
        )
        adapter = GeminiAdapter(api_key="dummy")
        with self.assertRaises(GenAIError) as ctx2:
            adapter._generate_body(req_google, model_name="gemini-1.5-flash")
        self.assertEqual(ctx2.exception.info.type, "InvalidRequestError")



import unittest
from contextlib import redirect_stderr
from contextlib import redirect_stdout
import io
from unittest.mock import patch


class TestCliSubcommands(unittest.TestCase):
    def test_mapping_subcommand(self) -> None:
        import gravtice.genai.cli as cli

        with patch.object(cli, "_print_mappings") as fn:
            cli.main(["mapping"])
            fn.assert_called_once()

    def test_model_sdk_subcommand(self) -> None:
        import gravtice.genai.cli as cli

        with patch.object(cli, "_print_sdk_supported") as fn:
            cli.main(["model", "sdk"])
            fn.assert_called_once()

    def test_model_available_requires_provider(self) -> None:
        import gravtice.genai.cli as cli

        with redirect_stderr(io.StringIO()):
            with self.assertRaises(SystemExit):
                cli.main(["model", "available"])

    def test_model_available_calls_handler(self) -> None:
        import gravtice.genai.cli as cli

        with patch.object(cli, "_print_available_models") as fn:
            cli.main(["model", "available", "--provider", "openai"])
            fn.assert_called_once()
            args, kwargs = fn.call_args
            self.assertEqual(args[0], "openai")
            self.assertIsNone(kwargs.get("timeout_ms"))

    def test_model_available_all_calls_handler(self) -> None:
        import gravtice.genai.cli as cli

        with patch.object(cli, "_print_all_available_models") as fn:
            cli.main(["model", "available", "--all"])
            fn.assert_called_once()

    def test_token_generate_prints_sk_token(self) -> None:
        import gravtice.genai.cli as cli

        buf = io.StringIO()
        with redirect_stdout(buf):
            cli.main(["token", "generate"])
        token = buf.getvalue().strip()
        self.assertTrue(token.startswith("sk-"))
        self.assertGreater(len(token), 10)



from __future__ import annotations

from typing_extensions import TypedDict


class CapabilityOverride(TypedDict, total=False):
    supports_stream: bool
    supports_job: bool


CAPABILITY_OVERRIDES: dict[str, dict[str, CapabilityOverride]] = {
    # NOTE: Keep this table small and explicit.
    #
    # Purpose:
    # - Fix known capability inference mistakes from heuristic `Adapter.capabilities(model_id)`.
    # - Provide a reviewable, auditable source of truth for edge models.
    #
    # Keying:
    # - provider: normalized lower-case provider name (e.g. "tuzi-web")
    # - model_id: exact catalog model id string (case-sensitive)
    #
    # Example:
    # "tuzi-web": {
    #     "some-model-id": {"supports_stream": False, "supports_job": True},
    # },
}



from __future__ import annotations

import errno
import http.client
import ipaddress
import json
import os
import socket
import ssl
import tempfile
import urllib.parse
from base64 import b64encode
from dataclasses import dataclass
from typing import Any, Iterable, Iterator
from uuid import uuid4

from .config import format_prefixed_env_names, get_default_timeout_ms, get_prefixed_env
from .errors import (
    auth_error,
    invalid_request_error,
    not_supported_error,
    provider_error,
    rate_limit_error,
    timeout_error,
)


def _timeout_seconds(timeout_ms: int | None) -> float:
    if timeout_ms is None:
        timeout_ms = get_default_timeout_ms()
    return max(0.001, timeout_ms / 1000.0)


def _env_truthy(name: str) -> bool:
    value = get_prefixed_env(name)
    return value in {"1", "true", "TRUE", "yes", "YES"}


def _default_url_download_max_bytes() -> int:
    raw = get_prefixed_env("URL_DOWNLOAD_MAX_BYTES")
    if raw is None:
        return 128 * 1024 * 1024
    try:
        value = int(raw)
    except ValueError:
        return 128 * 1024 * 1024
    return max(1, value)


def _is_private_ip(ip: ipaddress.IPv4Address | ipaddress.IPv6Address) -> bool:
    return bool(
        ip.is_private
        or ip.is_loopback
        or ip.is_link_local
        or ip.is_multicast
        or ip.is_reserved
        or ip.is_unspecified
    )


def _resolve_host_ips(host: str) -> list[ipaddress.IPv4Address | ipaddress.IPv6Address]:
    out: list[ipaddress.IPv4Address | ipaddress.IPv6Address] = []
    try:
        infos = socket.getaddrinfo(host, None, proto=socket.IPPROTO_TCP)
    except OSError:
        return out
    seen: set[ipaddress.IPv4Address | ipaddress.IPv6Address] = set()
    for family, _, _, _, sockaddr in infos:
        if family == socket.AF_INET:
            addr = sockaddr[0]
        elif family == socket.AF_INET6:
            addr = sockaddr[0]
        else:
            continue
        try:
            ip = ipaddress.ip_address(addr)
        except ValueError:
            continue
        if not isinstance(ip, (ipaddress.IPv4Address, ipaddress.IPv6Address)):
            continue
        if ip in seen:
            continue
        seen.add(ip)
        out.append(ip)
    return out


def _is_private_host(host: str) -> bool:
    h = host.strip().lower().rstrip(".")
    if h in {"localhost"} or h.endswith(".localhost"):
        return True
    try:
        ip = ipaddress.ip_address(h)
    except ValueError:
        ip = None
    if isinstance(ip, (ipaddress.IPv4Address, ipaddress.IPv6Address)):
        return _is_private_ip(ip)
    for resolved in _resolve_host_ips(h):
        if _is_private_ip(resolved):
            return True
    return False


def _resolve_url_host_ips(
    host: str,
) -> tuple[list[ipaddress.IPv4Address | ipaddress.IPv6Address], bool]:
    """
    Resolve a URL host once and classify it as private/loopback.

    Returns: (resolved_ips, is_private)
    """
    h = host.strip().lower().rstrip(".")
    if h in {"localhost"} or h.endswith(".localhost"):
        return [], True
    try:
        ip = ipaddress.ip_address(h)
    except ValueError:
        ip = None
    if isinstance(ip, (ipaddress.IPv4Address, ipaddress.IPv6Address)):
        return [ip], _is_private_ip(ip)
    resolved = _resolve_host_ips(h)
    return resolved, any(_is_private_ip(x) for x in resolved)


def download_to_file(
    *,
    url: str,
    output_path: str,
    timeout_ms: int | None = None,
    max_bytes: int | None = None,
    headers: dict[str, str] | None = None,
    proxy_url: str | None = None,
) -> None:
    """
    Download URL to a local file with timeout/proxy support and a hard size limit.

    Security: by default rejects obvious private/loopback IP hosts unless `GENAI_CALLING_ALLOW_PRIVATE_URLS=1`.
    """
    effective_max = (
        _default_url_download_max_bytes() if max_bytes is None else max_bytes
    )
    if effective_max <= 0:
        raise invalid_request_error("max_bytes must be positive")

    cur = url
    initial_host: str | None = None
    for _ in range(5):
        parsed = urllib.parse.urlparse(cur)
        if parsed.scheme.lower() not in {"http", "https"}:
            raise invalid_request_error(f"unsupported url scheme: {parsed.scheme}")
        if not parsed.hostname:
            raise invalid_request_error(f"invalid url: {cur}")
        if initial_host is None:
            initial_host = parsed.hostname
        resolved, is_private = _resolve_url_host_ips(parsed.hostname)
        if is_private and not _env_truthy("ALLOW_PRIVATE_URLS"):
            raise invalid_request_error(
                f"url host is private/loopback; set {format_prefixed_env_names('ALLOW_PRIVATE_URLS')}=1 to allow"
            )
        if not resolved:
            raise provider_error(
                f"dns resolution failed: {parsed.hostname}", retryable=True
            )
        connect_ip = str(resolved[0])

        path = _path_with_query(parsed)
        timeout_s = _timeout_seconds(timeout_ms)
        conn = _make_connection(
            parsed,
            timeout_s,
            proxy_url=proxy_url,
            connect_host=connect_ip,
            tls_server_hostname=parsed.hostname,
        )
        try:
            req_headers: dict[str, str] = {"Accept": "*/*"}
            if (
                headers
                and initial_host
                and parsed.hostname.lower() == initial_host.lower()
            ):
                req_headers.update(headers)
            if proxy_url:
                target_port = parsed.port or (
                    443 if parsed.scheme.lower() == "https" else 80
                )
                default_port = 443 if parsed.scheme.lower() == "https" else 80
                req_headers["Host"] = (
                    parsed.hostname
                    if target_port == default_port
                    else f"{parsed.hostname}:{target_port}"
                )
            conn.request("GET", path, headers=req_headers)
            resp = conn.getresponse()
            if resp.status in {301, 302, 303, 307, 308}:
                loc = resp.getheader("Location")
                if not loc:
                    raise provider_error("redirect response missing Location header")
                cur = urllib.parse.urljoin(cur, loc)
                continue
            if resp.status < 200 or resp.status >= 300:
                raw = resp.read(64 * 1024 + 1)
                _raise_for_status(resp.status, raw[: 64 * 1024])

            raw_len = resp.getheader("Content-Length")
            if raw_len:
                try:
                    n = int(raw_len)
                except ValueError:
                    n = -1
                if n > effective_max:
                    raise not_supported_error(
                        f"url download too large ({n} > {effective_max}); set {format_prefixed_env_names('URL_DOWNLOAD_MAX_BYTES')} or use path/ref"
                    )

            out_dir = os.path.dirname(os.path.abspath(output_path)) or "."
            with tempfile.NamedTemporaryFile(
                prefix="genaisdk-dl-", dir=out_dir, delete=False
            ) as tmp:
                tmp_path = tmp.name
            total = 0
            try:
                with open(tmp_path, "wb") as f:
                    while True:
                        chunk = resp.read(64 * 1024)
                        if not chunk:
                            break
                        total += len(chunk)
                        if total > effective_max:
                            raise not_supported_error(
                                f"url download exceeded limit ({total} > {effective_max}); set {format_prefixed_env_names('URL_DOWNLOAD_MAX_BYTES')} or use path/ref"
                            )
                        f.write(chunk)
            except Exception:
                try:
                    os.unlink(tmp_path)
                except OSError:
                    pass
                raise
            try:
                os.replace(tmp_path, output_path)
            except Exception:
                try:
                    os.unlink(tmp_path)
                except OSError:
                    pass
                raise
            return
        except (socket.timeout, TimeoutError):
            raise timeout_error("request timeout")
        except (ssl.SSLError, http.client.HTTPException, OSError) as e:
            raise provider_error(f"network error: {type(e).__name__}", retryable=True)
        finally:
            conn.close()

    raise provider_error("too many redirects", retryable=False)


def download_to_tempfile(
    *,
    url: str,
    timeout_ms: int | None = None,
    max_bytes: int | None = None,
    headers: dict[str, str] | None = None,
    suffix: str = "",
    proxy_url: str | None = None,
) -> str:
    with tempfile.NamedTemporaryFile(
        prefix="genaisdk-", suffix=suffix, delete=False
    ) as f:
        tmp_path = f.name
    try:
        download_to_file(
            url=url,
            output_path=tmp_path,
            timeout_ms=timeout_ms,
            max_bytes=max_bytes,
            headers=headers,
            proxy_url=proxy_url,
        )
    except Exception:
        try:
            os.unlink(tmp_path)
        except OSError:
            pass
        raise
    return tmp_path


def _proxy_tunnel_headers(proxy: urllib.parse.ParseResult) -> dict[str, str] | None:
    user = proxy.username
    pw = proxy.password
    if user is None and pw is None:
        return None
    user = "" if user is None else user
    pw = "" if pw is None else pw
    token = b64encode(f"{user}:{pw}".encode("utf-8")).decode("ascii")
    return {"Proxy-Authorization": f"Basic {token}"}


class _PinnedHTTPConnection(http.client.HTTPConnection):
    def __init__(
        self,
        host: str,
        port: int,
        *,
        connect_host: str,
        timeout: float,
    ) -> None:
        super().__init__(host, port, timeout=timeout)
        self._connect_host = connect_host

    def connect(self) -> None:
        self.sock = socket.create_connection(
            (self._connect_host, self.port), timeout=self.timeout
        )
        try:
            self.sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
        except OSError as e:
            if e.errno != errno.ENOPROTOOPT:
                raise
        if getattr(self, "_tunnel_host", None):
            self._tunnel()  # type: ignore[attr-defined]


class _PinnedHTTPSConnection(http.client.HTTPSConnection):
    def __init__(
        self,
        host: str,
        port: int,
        *,
        connect_host: str,
        tls_server_hostname: str,
        timeout: float,
        context: ssl.SSLContext,
    ) -> None:
        super().__init__(host, port, timeout=timeout, context=context)
        self._connect_host = connect_host
        self._tls_server_hostname = tls_server_hostname
        self._ssl_context = context

    def connect(self) -> None:
        self.sock = socket.create_connection(
            (self._connect_host, self.port), timeout=self.timeout
        )
        try:
            self.sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
        except OSError as e:
            if e.errno != errno.ENOPROTOOPT:
                raise
        if getattr(self, "_tunnel_host", None):
            self._tunnel()  # type: ignore[attr-defined]
        assert self.sock is not None
        self.sock = self._ssl_context.wrap_socket(
            self.sock,
            server_hostname=self._tls_server_hostname,
        )


def _make_connection(
    parsed: urllib.parse.ParseResult,
    timeout_s: float,
    *,
    proxy_url: str | None,
    connect_host: str | None = None,
    tls_server_hostname: str | None = None,
) -> http.client.HTTPConnection:
    scheme = parsed.scheme.lower()
    target_host = parsed.hostname
    if not target_host:
        raise invalid_request_error(f"invalid url: {parsed.geturl()}")

    target_port = parsed.port
    is_https = scheme == "https"
    if not is_https and scheme != "http":
        raise invalid_request_error(f"unsupported url scheme: {scheme}")

    target_connect_host = target_host if connect_host is None else connect_host
    tls_hostname = target_host if tls_server_hostname is None else tls_server_hostname

    if proxy_url:
        p = urllib.parse.urlparse(proxy_url)
        if not p.hostname:
            raise invalid_request_error(f"invalid proxy url: {proxy_url}")
        if p.scheme.lower() not in {"http", "https"}:
            raise invalid_request_error(f"unsupported proxy url scheme: {p.scheme}")
        proxy_port = p.port or (443 if p.scheme == "https" else 80)
        effective_target_port = target_port or (443 if is_https else 80)
        if is_https:
            ctx = ssl.create_default_context()
            conn: http.client.HTTPConnection = _PinnedHTTPSConnection(
                p.hostname,
                proxy_port,
                connect_host=p.hostname,
                tls_server_hostname=tls_hostname,
                timeout=timeout_s,
                context=ctx,
            )
        else:
            conn = http.client.HTTPConnection(p.hostname, proxy_port, timeout=timeout_s)
        conn.set_tunnel(
            target_connect_host, effective_target_port, headers=_proxy_tunnel_headers(p)
        )
        return conn

    if is_https:
        ctx = ssl.create_default_context()
        effective_port = target_port or 443
        if target_connect_host != target_host or tls_hostname != target_host:
            return _PinnedHTTPSConnection(
                target_host,
                effective_port,
                connect_host=target_connect_host,
                tls_server_hostname=tls_hostname,
                timeout=timeout_s,
                context=ctx,
            )
        return http.client.HTTPSConnection(
            target_host, effective_port, timeout=timeout_s, context=ctx
        )
    effective_port = target_port or 80
    if target_connect_host != target_host:
        return _PinnedHTTPConnection(
            target_host,
            effective_port,
            connect_host=target_connect_host,
            timeout=timeout_s,
        )
    return http.client.HTTPConnection(target_host, effective_port, timeout=timeout_s)


def _path_with_query(parsed: urllib.parse.ParseResult) -> str:
    path = parsed.path or "/"
    if parsed.query:
        return f"{path}?{parsed.query}"
    return path


def _extract_error_message(body: bytes) -> tuple[str, str | None]:
    if not body:
        return "empty error body", None
    try:
        obj = json.loads(body)
    except Exception:
        text = body.decode("utf-8", errors="replace")
        return text[:2_000], None

    if isinstance(obj, dict):
        if isinstance(obj.get("error"), dict):
            err = obj["error"]
            msg = err.get("message") or err.get("detail") or str(err)
            code = err.get("code") or err.get("status") or None
            return str(msg)[:2_000], str(code) if code is not None else None
        msg = obj.get("message") or str(obj)
        code = obj.get("code") or obj.get("status") or None
        return str(msg)[:2_000], str(code) if code is not None else None

    return str(obj)[:2_000], None


def _raise_for_status(status: int, body: bytes) -> None:
    message, provider_code = _extract_error_message(body)
    if status in (401, 403):
        raise auth_error(message, provider_code=provider_code)
    if status == 429:
        raise rate_limit_error(message, provider_code=provider_code)
    if status in (400, 404, 409, 415, 422):
        raise invalid_request_error(message, provider_code=provider_code)
    if status in (408, 504):
        raise timeout_error(message)
    if 500 <= status <= 599:
        raise provider_error(message, provider_code=provider_code, retryable=True)
    raise provider_error(message, provider_code=provider_code, retryable=False)


@dataclass(frozen=True, slots=True)
class StreamingBody:
    content_type: str
    content_length: int
    chunks: Iterable[bytes]


def multipart_form_data_fields(*, fields: dict[str, str]) -> StreamingBody:
    boundary = uuid4().hex
    boundary_bytes = boundary.encode("ascii")
    crlf = b"\r\n"
    tail = b"--" + boundary_bytes + b"--" + crlf

    parts = []
    total = len(tail)
    for k, v in fields.items():
        part = (
            b"--"
            + boundary_bytes
            + crlf
            + f'Content-Disposition: form-data; name="{k}"'.encode("utf-8")
            + crlf
            + crlf
            + v.encode("utf-8")
            + crlf
        )
        parts.append(part)
        total += len(part)

    def _chunks() -> Iterator[bytes]:
        yield from parts
        yield tail

    return StreamingBody(
        content_type=f"multipart/form-data; boundary={boundary}",
        content_length=total,
        chunks=_chunks(),
    )


def multipart_form_data(
    *,
    fields: dict[str, str],
    file_field: str,
    file_path: str,
    filename: str,
    file_mime_type: str,
    chunk_size: int = 64 * 1024,
) -> StreamingBody:
    boundary = uuid4().hex
    boundary_bytes = boundary.encode("ascii")
    crlf = b"\r\n"

    def _field_part(name: str, value: str) -> bytes:
        header = (
            b"--"
            + boundary_bytes
            + crlf
            + f'Content-Disposition: form-data; name="{name}"'.encode("utf-8")
            + crlf
            + crlf
        )
        return header + value.encode("utf-8") + crlf

    def _file_preamble() -> bytes:
        return (
            b"--"
            + boundary_bytes
            + crlf
            + (
                f'Content-Disposition: form-data; name="{file_field}"; filename="{filename}"'
            ).encode("utf-8")
            + crlf
            + f"Content-Type: {file_mime_type}".encode("utf-8")
            + crlf
            + crlf
        )

    file_size = os.stat(file_path).st_size
    tail = b"--" + boundary_bytes + b"--" + crlf
    preamble = _file_preamble()

    total = len(tail) + len(preamble) + file_size + len(crlf)
    field_parts = []
    for k, v in fields.items():
        part = _field_part(k, v)
        field_parts.append(part)
        total += len(part)

    def _chunks() -> Iterator[bytes]:
        for part in field_parts:
            yield part
        yield preamble
        with open(file_path, "rb") as f:
            while True:
                chunk = f.read(chunk_size)
                if not chunk:
                    break
                yield chunk
        yield crlf
        yield tail

    return StreamingBody(
        content_type=f"multipart/form-data; boundary={boundary}",
        content_length=total,
        chunks=_chunks(),
    )


def multipart_form_data_json_and_file(
    *,
    metadata_field: str,
    metadata: dict[str, Any],
    file_field: str,
    file_path: str,
    filename: str,
    file_mime_type: str,
    chunk_size: int = 64 * 1024,
) -> StreamingBody:
    boundary = uuid4().hex
    boundary_bytes = boundary.encode("ascii")
    crlf = b"\r\n"

    meta_bytes = json.dumps(metadata, separators=(",", ":")).encode("utf-8")
    meta_part = (
        b"--"
        + boundary_bytes
        + crlf
        + f'Content-Disposition: form-data; name="{metadata_field}"'.encode("utf-8")
        + crlf
        + b"Content-Type: application/json; charset=utf-8"
        + crlf
        + crlf
        + meta_bytes
        + crlf
    )
    file_preamble = (
        b"--"
        + boundary_bytes
        + crlf
        + (
            f'Content-Disposition: form-data; name="{file_field}"; filename="{filename}"'
        ).encode("utf-8")
        + crlf
        + f"Content-Type: {file_mime_type}".encode("utf-8")
        + crlf
        + crlf
    )
    tail = b"--" + boundary_bytes + b"--" + crlf

    file_size = os.stat(file_path).st_size
    total = len(meta_part) + len(file_preamble) + file_size + len(crlf) + len(tail)

    def _chunks() -> Iterator[bytes]:
        yield meta_part
        yield file_preamble
        with open(file_path, "rb") as f:
            while True:
                chunk = f.read(chunk_size)
                if not chunk:
                    break
                yield chunk
        yield crlf
        yield tail

    return StreamingBody(
        content_type=f"multipart/form-data; boundary={boundary}",
        content_length=total,
        chunks=_chunks(),
    )


def request_json(
    *,
    method: str,
    url: str,
    headers: dict[str, str] | None = None,
    json_body: Any | None = None,
    timeout_ms: int | None = None,
    proxy_url: str | None = None,
) -> dict[str, Any]:
    body = (
        None
        if json_body is None
        else json.dumps(json_body, separators=(",", ":")).encode("utf-8")
    )
    req_headers = {"Accept": "application/json"}
    if body is not None:
        req_headers["Content-Type"] = "application/json"
        req_headers["Content-Length"] = str(len(body))
    if headers:
        req_headers.update(headers)

    parsed = urllib.parse.urlparse(url)
    path = _path_with_query(parsed)
    timeout_s = _timeout_seconds(timeout_ms)
    conn = _make_connection(parsed, timeout_s, proxy_url=proxy_url)
    try:
        conn.request(method.upper(), path, body=body, headers=req_headers)
        resp = conn.getresponse()
        raw = resp.read()
        if resp.status < 200 or resp.status >= 300:
            _raise_for_status(resp.status, raw)
        if not raw:
            return {}
        try:
            obj = json.loads(raw)
        except Exception:
            raise provider_error("invalid json response", retryable=True)
        if not isinstance(obj, dict):
            raise provider_error("invalid json response", retryable=True)
        return obj
    except (socket.timeout, TimeoutError):
        raise timeout_error("request timeout")
    except (ssl.SSLError, http.client.HTTPException, OSError) as e:
        raise provider_error(f"network error: {type(e).__name__}", retryable=True)
    finally:
        conn.close()


def request_bytes(
    *,
    method: str,
    url: str,
    headers: dict[str, str] | None = None,
    body: bytes | None = None,
    timeout_ms: int | None = None,
    proxy_url: str | None = None,
) -> bytes:
    req_headers: dict[str, str] = {}
    if body is not None:
        req_headers["Content-Length"] = str(len(body))
    if headers:
        req_headers.update(headers)

    parsed = urllib.parse.urlparse(url)
    path = _path_with_query(parsed)
    timeout_s = _timeout_seconds(timeout_ms)
    conn = _make_connection(parsed, timeout_s, proxy_url=proxy_url)
    try:
        conn.request(method.upper(), path, body=body, headers=req_headers)
        resp = conn.getresponse()
        raw = resp.read()
        if resp.status < 200 or resp.status >= 300:
            _raise_for_status(resp.status, raw)
        return raw
    except (socket.timeout, TimeoutError):
        raise timeout_error("request timeout")
    except (ssl.SSLError, http.client.HTTPException, OSError) as e:
        raise provider_error(f"network error: {type(e).__name__}", retryable=True)
    finally:
        conn.close()


def request_stream_json_sse(
    *,
    method: str,
    url: str,
    headers: dict[str, str] | None = None,
    json_body: Any | None = None,
    timeout_ms: int | None = None,
    proxy_url: str | None = None,
) -> Iterator[dict[str, Any]]:
    body = (
        None
        if json_body is None
        else json.dumps(json_body, separators=(",", ":")).encode("utf-8")
    )
    req_headers = {"Accept": "text/event-stream"}
    if body is not None:
        req_headers["Content-Type"] = "application/json"
        req_headers["Content-Length"] = str(len(body))
    if headers:
        req_headers.update(headers)

    parsed = urllib.parse.urlparse(url)
    path = _path_with_query(parsed)
    timeout_s = _timeout_seconds(timeout_ms)
    conn = _make_connection(parsed, timeout_s, proxy_url=proxy_url)

    try:
        conn.request(method.upper(), path, body=body, headers=req_headers)
        resp = conn.getresponse()
        if resp.status < 200 or resp.status >= 300:
            raw = resp.read()
            _raise_for_status(resp.status, raw)

        def _iter() -> Iterator[dict[str, Any]]:
            try:
                for ev in _iter_sse_events(resp):
                    if not ev.data:
                        continue
                    if ev.data == "[DONE]":
                        return
                    try:
                        obj = json.loads(ev.data)
                    except Exception:
                        raise provider_error(f"invalid sse json: {ev.data[:200]}")
                    if isinstance(obj, dict):
                        yield obj
            finally:
                conn.close()

        return _iter()
    except (socket.timeout, TimeoutError):
        conn.close()
        raise timeout_error("request timeout")
    except (ssl.SSLError, http.client.HTTPException, OSError) as e:
        conn.close()
        raise provider_error(f"network error: {type(e).__name__}", retryable=True)
    except Exception:
        conn.close()
        raise


def request_streaming_body_json(
    *,
    method: str,
    url: str,
    headers: dict[str, str] | None,
    body: StreamingBody,
    timeout_ms: int | None = None,
    proxy_url: str | None = None,
) -> dict[str, Any]:
    req_headers = {"Accept": "application/json"}
    req_headers["Content-Type"] = body.content_type
    req_headers["Content-Length"] = str(body.content_length)
    if headers:
        req_headers.update(headers)

    parsed = urllib.parse.urlparse(url)
    path = _path_with_query(parsed)
    timeout_s = _timeout_seconds(timeout_ms)
    conn = _make_connection(parsed, timeout_s, proxy_url=proxy_url)

    try:
        conn.putrequest(method.upper(), path)
        for k, v in req_headers.items():
            conn.putheader(k, v)
        conn.endheaders()
        for chunk in body.chunks:
            if chunk:
                conn.send(chunk)
        resp = conn.getresponse()
        raw = resp.read()
        if resp.status < 200 or resp.status >= 300:
            _raise_for_status(resp.status, raw)
        if not raw:
            return {}
        try:
            obj = json.loads(raw)
        except Exception:
            raise provider_error("invalid json response", retryable=True)
        if not isinstance(obj, dict):
            raise provider_error("invalid json response", retryable=True)
        return obj
    except (socket.timeout, TimeoutError):
        raise timeout_error("request timeout")
    except (ssl.SSLError, http.client.HTTPException, OSError) as e:
        raise provider_error(f"network error: {type(e).__name__}", retryable=True)
    finally:
        conn.close()


@dataclass(frozen=True, slots=True)
class SSEEvent:
    data: str
    event: str | None = None
    id: str | None = None
    retry: int | None = None


def request_stream_sse(
    *,
    method: str,
    url: str,
    headers: dict[str, str] | None = None,
    body: bytes | None = None,
    timeout_ms: int | None = None,
    proxy_url: str | None = None,
) -> Iterator[SSEEvent]:
    req_headers = {"Accept": "text/event-stream"}
    if body is not None:
        req_headers["Content-Length"] = str(len(body))
    if headers:
        req_headers.update(headers)

    parsed = urllib.parse.urlparse(url)
    path = _path_with_query(parsed)
    timeout_s = _timeout_seconds(timeout_ms)
    conn = _make_connection(parsed, timeout_s, proxy_url=proxy_url)

    try:
        conn.request(method.upper(), path, body=body, headers=req_headers)
        resp = conn.getresponse()
        if resp.status < 200 or resp.status >= 300:
            raw = resp.read()
            _raise_for_status(resp.status, raw)

        def _iter() -> Iterator[SSEEvent]:
            try:
                yield from _iter_sse_events(resp)
            finally:
                conn.close()

        return _iter()
    except (socket.timeout, TimeoutError):
        conn.close()
        raise timeout_error("request timeout")
    except (ssl.SSLError, http.client.HTTPException, OSError) as e:
        conn.close()
        raise provider_error(f"network error: {type(e).__name__}", retryable=True)
    except Exception:
        conn.close()
        raise


def _iter_sse_events(resp: http.client.HTTPResponse) -> Iterator[SSEEvent]:
    buffer: list[str] = []
    event_type: str | None = None
    event_id: str | None = None
    retry_ms: int | None = None
    while True:
        line = resp.readline()
        if not line:
            break
        text = line.decode("utf-8", errors="replace")
        text = text.rstrip("\n")
        if text.endswith("\r"):
            text = text[:-1]
        if not text:
            if (
                buffer
                or event_type is not None
                or event_id is not None
                or retry_ms is not None
            ):
                yield SSEEvent(
                    data="\n".join(buffer),
                    event=event_type,
                    id=event_id,
                    retry=retry_ms,
                )
                buffer.clear()
                event_type = None
                event_id = None
                retry_ms = None
            continue
        if text.startswith(":"):
            continue
        if ":" in text:
            field, value = text.split(":", 1)
            value = value[1:] if value.startswith(" ") else value
        else:
            field, value = text, ""
        if field == "data":
            buffer.append(value)
        elif field == "event":
            event_type = value or None
        elif field == "id":
            if "\x00" not in value:
                event_id = value
        elif field == "retry":
            try:
                retry_ms = int(value)
            except ValueError:
                continue
    if buffer or event_type is not None or event_id is not None or retry_ms is not None:
        yield SSEEvent(
            data="\n".join(buffer), event=event_type, id=event_id, retry=retry_ms
        )


def _iter_sse_data(resp: http.client.HTTPResponse) -> Iterator[str]:
    for ev in _iter_sse_events(resp):
        if ev.data:
            yield ev.data



import json
import unittest
from unittest.mock import patch


class TestToolCalling(unittest.TestCase):
    def test_openai_chat_tools_and_tool_choice(self) -> None:
        from gravtice.genai.types import (
            GenerateRequest,
            Message,
            OutputSpec,
            Part,
            Tool,
            ToolChoice,
        )
        from gravtice.genai.providers.openai import OpenAIAdapter

        req = GenerateRequest(
            model="openai:gpt-4o-mini",
            input=[
                Message(role="user", content=[Part.from_text("hi")]),
                Message(
                    role="assistant",
                    content=[
                        Part.tool_call(
                            tool_call_id="call_1",
                            name="sum",
                            arguments={"a": 1, "b": 2},
                        ),
                    ],
                ),
                Message(
                    role="tool",
                    content=[
                        Part.tool_result(
                            tool_call_id="call_1", name="sum", result={"value": 3}
                        )
                    ],
                ),
            ],
            output=OutputSpec(modalities=["text"]),
            tools=[
                Tool(
                    name="sum",
                    description="sum two integers",
                    parameters={
                        "type": "object",
                        "properties": {
                            "a": {"type": "integer"},
                            "b": {"type": "integer"},
                        },
                        "required": ["a", "b"],
                    },
                    strict=True,
                )
            ],
            tool_choice=ToolChoice(mode="tool", name="sum"),
        )

        with patch("gravtice.genai.providers.openai.request_json") as request_json:
            request_json.return_value = {
                "id": "chatcmpl_123",
                "choices": [
                    {
                        "message": {
                            "tool_calls": [
                                {
                                    "id": "call_2",
                                    "type": "function",
                                    "function": {
                                        "name": "sum",
                                        "arguments": '{"a":2,"b":3}',
                                    },
                                }
                            ]
                        }
                    }
                ],
            }
            adapter = OpenAIAdapter(
                api_key="__demo__",
                base_url="https://example.invalid/v1",
                provider_name="openai",
            )
            out = adapter.generate(req, stream=False)
            self.assertEqual(out.status, "completed")
            self.assertEqual(out.output[0].content[0].type, "tool_call")
            self.assertEqual(
                out.output[0].content[0].meta.get("tool_call_id"), "call_2"
            )
            self.assertEqual(out.output[0].content[0].meta.get("name"), "sum")
            self.assertEqual(
                out.output[0].content[0].meta.get("arguments"), {"a": 2, "b": 3}
            )

            _, kwargs = request_json.call_args
            body = kwargs["json_body"]
            self.assertEqual(body["tool_choice"]["function"]["name"], "sum")
            self.assertEqual(body["tools"][0]["function"]["name"], "sum")
            self.assertEqual(body["tools"][0]["function"]["strict"], True)

            msgs = body["messages"]
            self.assertEqual(msgs[1]["tool_calls"][0]["id"], "call_1")
            self.assertEqual(
                msgs[1]["tool_calls"][0]["function"]["arguments"],
                json.dumps({"a": 1, "b": 2}, separators=(",", ":"), ensure_ascii=False),
            )
            self.assertEqual(msgs[2]["role"], "tool")
            self.assertEqual(msgs[2]["tool_call_id"], "call_1")
            self.assertEqual(
                msgs[2]["content"],
                json.dumps({"value": 3}, separators=(",", ":"), ensure_ascii=False),
            )

    def test_openai_responses_tools_and_tool_output(self) -> None:
        from gravtice.genai.types import (
            GenerateRequest,
            Message,
            OutputSpec,
            Part,
            Tool,
            ToolChoice,
        )
        from gravtice.genai.providers.openai import OpenAIAdapter

        req = GenerateRequest(
            model="openai:gpt-5.2",
            input=[
                Message(role="user", content=[Part.from_text("hi")]),
                Message(
                    role="tool",
                    content=[
                        Part.tool_result(
                            tool_call_id="call_1", name="sum", result={"value": 3}
                        )
                    ],
                ),
            ],
            output=OutputSpec(modalities=["text"]),
            tools=[Tool(name="sum", parameters={"type": "object"})],
            tool_choice=ToolChoice(mode="required"),
        )

        with patch("gravtice.genai.providers.openai.request_json") as request_json:
            request_json.return_value = {
                "id": "resp_123",
                "status": "completed",
                "output": [
                    {
                        "type": "function_call",
                        "call_id": "call_2",
                        "name": "sum",
                        "arguments": '{"a":2,"b":3}',
                    },
                    {
                        "type": "message",
                        "content": [{"type": "output_text", "text": "ok"}],
                    },
                ],
            }
            adapter = OpenAIAdapter(
                api_key="__demo__",
                base_url="https://example.invalid/v1",
                provider_name="openai",
                chat_api="responses",
            )
            out = adapter.generate(req, stream=False)
            self.assertEqual(out.status, "completed")
            self.assertEqual(
                [p.type for p in out.output[0].content], ["tool_call", "text"]
            )
            self.assertEqual(
                out.output[0].content[0].meta.get("tool_call_id"), "call_2"
            )
            self.assertEqual(
                out.output[0].content[0].meta.get("arguments"), {"a": 2, "b": 3}
            )

            _, kwargs = request_json.call_args
            body = kwargs["json_body"]
            self.assertEqual(body["tool_choice"], "required")
            self.assertEqual(body["tools"][0]["type"], "function")
            self.assertEqual(body["tools"][0]["name"], "sum")
            self.assertNotIn("strict", body["tools"][0])

            items = body["input"]
            self.assertEqual(items[1]["type"], "function_call_output")
            self.assertEqual(items[1]["call_id"], "call_1")
            self.assertEqual(
                items[1]["output"],
                json.dumps({"value": 3}, separators=(",", ":"), ensure_ascii=False),
            )

    def test_tuzi_openai_responses_tool_choice_tool_falls_back_to_required(
        self,
    ) -> None:
        from gravtice.genai.types import (
            GenerateRequest,
            Message,
            OutputSpec,
            Part,
            Tool,
            ToolChoice,
        )
        from gravtice.genai.providers.openai import OpenAIAdapter

        req = GenerateRequest(
            model="tuzi-openai:gpt-4o-mini",
            input=[Message(role="user", content=[Part.from_text("hi")])],
            output=OutputSpec(modalities=["text"]),
            tools=[Tool(name="ping", parameters={"type": "object", "properties": {}})],
            tool_choice=ToolChoice(mode="tool", name="ping"),
        )

        with patch("gravtice.genai.providers.openai.request_json") as request_json:
            request_json.return_value = {
                "id": "resp_123",
                "status": "completed",
                "output": [
                    {
                        "type": "message",
                        "content": [{"type": "output_text", "text": "ok"}],
                    }
                ],
            }
            adapter = OpenAIAdapter(
                api_key="__demo__",
                base_url="https://example.invalid/v1",
                provider_name="tuzi-openai",
                chat_api="responses",
            )
            out = adapter.generate(req, stream=False)
            self.assertEqual(out.status, "completed")

            _, kwargs = request_json.call_args
            body = kwargs["json_body"]
            self.assertEqual(body["tool_choice"], "required")

    def test_gemini_tools_and_tool_config(self) -> None:
        from gravtice.genai.types import (
            GenerateRequest,
            Message,
            OutputSpec,
            Part,
            Tool,
            ToolChoice,
        )
        from gravtice.genai.providers.gemini import GeminiAdapter

        req = GenerateRequest(
            model="google:gemini-2.5-flash",
            input=[
                Message(role="user", content=[Part.from_text("hi")]),
                Message(
                    role="tool",
                    content=[Part.tool_result(name="sum", result={"value": 3})],
                ),
            ],
            output=OutputSpec(modalities=["text"]),
            tools=[Tool(name="sum", parameters={"type": "object"})],
            tool_choice=ToolChoice(mode="tool", name="sum"),
        )

        with patch("gravtice.genai.providers.gemini.request_json") as request_json:
            request_json.return_value = {
                "candidates": [
                    {
                        "content": {
                            "parts": [
                                {
                                    "functionCall": {
                                        "name": "sum",
                                        "args": {"a": 1, "b": 2},
                                    }
                                }
                            ]
                        }
                    }
                ]
            }
            adapter = GeminiAdapter(
                api_key="__demo__",
                base_url="https://example.invalid",
                provider_name="google",
            )
            out = adapter.generate(req, stream=False)
            self.assertEqual(out.status, "completed")
            self.assertEqual(out.output[0].content[0].type, "tool_call")
            self.assertEqual(out.output[0].content[0].meta.get("name"), "sum")
            self.assertEqual(
                out.output[0].content[0].meta.get("arguments"), {"a": 1, "b": 2}
            )

            _, kwargs = request_json.call_args
            body = kwargs["json_body"]
            self.assertEqual(body["tools"][0]["functionDeclarations"][0]["name"], "sum")
            self.assertEqual(body["toolConfig"]["functionCallingConfig"]["mode"], "ANY")
            self.assertEqual(
                body["toolConfig"]["functionCallingConfig"]["allowedFunctionNames"],
                ["sum"],
            )

            contents = body["contents"]
            self.assertEqual(contents[1]["role"], "user")
            self.assertEqual(contents[1]["parts"][0]["functionResponse"]["name"], "sum")

    def test_anthropic_tools_and_tool_choice(self) -> None:
        from gravtice.genai.types import (
            GenerateRequest,
            Message,
            OutputSpec,
            OutputTextSpec,
            Part,
            Tool,
            ToolChoice,
        )
        from gravtice.genai.providers.anthropic import AnthropicAdapter

        req = GenerateRequest(
            model="anthropic:claude-3-5-sonnet-20240620",
            input=[
                Message(role="user", content=[Part.from_text("hi")]),
                Message(
                    role="tool",
                    content=[
                        Part.tool_result(
                            tool_call_id="toolu_1", name="sum", result={"value": 3}
                        )
                    ],
                ),
            ],
            output=OutputSpec(
                modalities=["text"], text=OutputTextSpec(max_output_tokens=256)
            ),
            tools=[Tool(name="sum", parameters={"type": "object"})],
            tool_choice=ToolChoice(mode="tool", name="sum"),
        )

        with patch("gravtice.genai.providers.anthropic.request_json") as request_json:
            request_json.return_value = {
                "id": "msg_123",
                "content": [
                    {
                        "type": "tool_use",
                        "id": "toolu_2",
                        "name": "sum",
                        "input": {"a": 1, "b": 2},
                    }
                ],
            }
            adapter = AnthropicAdapter(
                api_key="__demo__",
                base_url="https://example.invalid",
                provider_name="anthropic",
            )
            out = adapter.generate(req, stream=False)
            self.assertEqual(out.status, "completed")
            self.assertEqual(out.output[0].content[0].type, "tool_call")
            self.assertEqual(
                out.output[0].content[0].meta.get("tool_call_id"), "toolu_2"
            )
            self.assertEqual(
                out.output[0].content[0].meta.get("arguments"), {"a": 1, "b": 2}
            )

            _, kwargs = request_json.call_args
            body = kwargs["json_body"]
            self.assertEqual(body["tools"][0]["name"], "sum")
            self.assertEqual(body["tool_choice"], {"type": "tool", "name": "sum"})

            msgs = body["messages"]
            tool_msg = msgs[1]
            self.assertEqual(tool_msg["role"], "user")
            self.assertEqual(tool_msg["content"][0]["type"], "tool_result")
            self.assertEqual(tool_msg["content"][0]["tool_use_id"], "toolu_1")



import unittest
from unittest.mock import patch


class _FakeResponse:
    def __init__(self, *, status: int, body: bytes) -> None:
        self.status = status
        self._body = body

    def read(self, n: int | None = None) -> bytes:  # noqa: ARG002
        return self._body


class _FakeConn:
    def __init__(self, resp: _FakeResponse) -> None:
        self._resp = resp

    def request(
        self,
        method: str,
        path: str,
        body: object = None,
        headers: dict[str, str] | None = None,
    ):  # noqa: ARG002
        return

    def getresponse(self) -> _FakeResponse:
        return self._resp

    def close(self) -> None:
        return


class TestHttpErrorMapping(unittest.TestCase):
    def test_request_json_maps_500_to_retryable_provider_error(self) -> None:
        from gravtice.genai import GenAIError
        from gravtice.genai._internal.http import request_json

        resp = _FakeResponse(status=500, body=b'{"message":"oops"}')

        def _fake_make_connection(
            parsed, timeout_s, *, proxy_url, connect_host=None, tls_server_hostname=None
        ):  # type: ignore[no-untyped-def]
            return _FakeConn(resp)

        with patch(
            "gravtice.genai._internal.http._make_connection",
            side_effect=_fake_make_connection,
        ):
            with self.assertRaises(GenAIError) as cm:
                request_json(method="GET", url="https://example.com/x")

        self.assertEqual(cm.exception.info.type, "ProviderError")
        self.assertTrue(cm.exception.info.retryable)

    def test_request_json_maps_429_to_rate_limit_error(self) -> None:
        from gravtice.genai import GenAIError
        from gravtice.genai._internal.http import request_json

        resp = _FakeResponse(status=429, body=b'{"message":"slow down"}')

        def _fake_make_connection(
            parsed, timeout_s, *, proxy_url, connect_host=None, tls_server_hostname=None
        ):  # type: ignore[no-untyped-def]
            return _FakeConn(resp)

        with patch(
            "gravtice.genai._internal.http._make_connection",
            side_effect=_fake_make_connection,
        ):
            with self.assertRaises(GenAIError) as cm:
                request_json(method="GET", url="https://example.com/x")

        self.assertEqual(cm.exception.info.type, "RateLimitError")
        self.assertTrue(cm.exception.info.retryable)

    def test_request_json_maps_network_error_to_retryable_provider_error(self) -> None:
        from gravtice.genai import GenAIError
        from gravtice.genai._internal.http import request_json

        class _NetErrConn:
            def request(
                self,
                method: str,
                path: str,
                body: object = None,
                headers: dict[str, str] | None = None,
            ):  # noqa: ARG002
                raise OSError("boom")

            def close(self) -> None:
                return

        def _fake_make_connection(
            parsed, timeout_s, *, proxy_url, connect_host=None, tls_server_hostname=None
        ):  # type: ignore[no-untyped-def]
            return _NetErrConn()

        with patch(
            "gravtice.genai._internal.http._make_connection",
            side_effect=_fake_make_connection,
        ):
            with self.assertRaises(GenAIError) as cm:
                request_json(method="GET", url="https://example.com/x")

        self.assertEqual(cm.exception.info.type, "ProviderError")
        self.assertTrue(cm.exception.info.retryable)



import unittest
from unittest.mock import patch


class TestTuziModels(unittest.TestCase):
    def test_tuzi_catalog_has_no_duplicates(self) -> None:
        from gravtice.genai.reference.catalog import MODEL_CATALOG

        for provider in ("tuzi-web", "tuzi-openai", "tuzi-google", "tuzi-anthropic"):
            model_ids = MODEL_CATALOG[provider]
            with self.subTest(provider=provider):
                self.assertEqual(len(model_ids), len(set(model_ids)))

    def test_tuzi_web_routes_by_model_id_prefix(self) -> None:
        from gravtice.genai.providers import TuziAdapter

        openai = object()
        gemini = object()
        anthropic = object()
        tuzi = TuziAdapter(openai=openai, gemini=gemini, anthropic=anthropic)

        self.assertIs(tuzi._route("gpt-4o-mini"), openai)
        self.assertIs(tuzi._route("sora-2"), openai)
        self.assertIs(tuzi._route("claude-sonnet-4-5"), anthropic)
        self.assertIs(tuzi._route("gemini-2.5-flash"), gemini)
        self.assertIs(tuzi._route("gemma-3-4b-it"), gemini)
        self.assertIs(tuzi._route("veo-3.0-generate"), gemini)
        self.assertIs(tuzi._route("text-embedding-004"), gemini)

    def test_tuzi_web_route_requires_protocol_keys(self) -> None:
        from gravtice.genai import GenAIError
        from gravtice.genai.providers import TuziAdapter

        tuzi = TuziAdapter(openai=None, gemini=None, anthropic=None)

        with self.assertRaises(GenAIError) as cm:
            tuzi._route("gpt-4o-mini")
        self.assertEqual(cm.exception.info.type, "InvalidRequestError")

        with self.assertRaises(GenAIError) as cm:
            tuzi._route("gemini-2.5-flash")
        self.assertEqual(cm.exception.info.type, "InvalidRequestError")

        with self.assertRaises(GenAIError) as cm:
            tuzi._route("claude-3-sonnet-20240229")
        self.assertEqual(cm.exception.info.type, "InvalidRequestError")

    def test_tuzi_sora_seconds_sent_as_string(self) -> None:
        from gravtice.genai.types import (
            GenerateRequest,
            Message,
            OutputSpec,
            OutputVideoSpec,
            Part,
        )
        from gravtice.genai.providers.openai import OpenAIAdapter
        from gravtice.genai._internal.http import (
            multipart_form_data_fields as real_multipart_form_data_fields,
        )

        req = GenerateRequest(
            model="tuzi-openai:sora-2",
            input=[Message(role="user", content=[Part.from_text("test")])],
            output=OutputSpec(
                modalities=["video"],
                video=OutputVideoSpec(duration_sec=4, aspect_ratio="16:9"),
            ),
            wait=False,
        )

        with (
            patch(
                "gravtice.genai.providers.openai.multipart_form_data_fields"
            ) as multipart_form_data_fields,
            patch(
                "gravtice.genai.providers.openai.request_streaming_body_json"
            ) as request_streaming_body_json,
        ):
            multipart_form_data_fields.side_effect = real_multipart_form_data_fields
            request_streaming_body_json.return_value = {"id": "vid_123"}
            adapter = OpenAIAdapter(
                api_key="__demo__",
                base_url="https://example.invalid/v1",
                provider_name="tuzi-openai",
            )
            out = adapter.generate(req, stream=False)
            self.assertEqual(out.status, "running")

            _, kwargs = multipart_form_data_fields.call_args
            fields = kwargs["fields"]
            self.assertIsInstance(fields.get("seconds"), str)
            self.assertEqual(fields.get("seconds"), "4")

        with patch("gravtice.genai.providers.openai.request_json") as request_json:
            request_json.return_value = {"id": "vid_123"}
            adapter = OpenAIAdapter(
                api_key="__demo__",
                base_url="https://example.invalid/v1",
                provider_name="openai",
            )
            out = adapter.generate(req, stream=False)
            self.assertEqual(out.status, "running")

            _, kwargs = request_json.call_args
            body = kwargs["json_body"]
            self.assertIsInstance(body.get("seconds"), int)

    def test_openai_adapter_capabilities_infer_tuzi_modalities(self) -> None:
        from gravtice.genai.providers.openai import OpenAIAdapter

        adapter = OpenAIAdapter(
            api_key="__demo__",
            base_url="https://example.invalid/v1",
            provider_name="tuzi-openai",
        )

        self.assertEqual(
            adapter.capabilities("sd3.5-large").output_modalities, {"image"}
        )
        self.assertEqual(adapter.capabilities("pika-1.5").output_modalities, {"video"})
        self.assertEqual(
            adapter.capabilities("chirp-v3-5").output_modalities, {"audio"}
        )

        cap = adapter.capabilities("whisper-large-v3")
        self.assertEqual(cap.input_modalities, {"audio"})
        self.assertEqual(cap.output_modalities, {"text"})

    def test_tuzi_web_chirp_v3_5_uses_suno_submit_music(self) -> None:
        from gravtice.genai.providers import TuziAdapter
        from gravtice.genai.providers.openai import OpenAIAdapter
        from gravtice.genai.types import (
            GenerateRequest,
            Message,
            OutputAudioSpec,
            OutputSpec,
            Part,
        )

        tuzi = TuziAdapter(
            openai=OpenAIAdapter(
                api_key="__demo__",
                base_url="https://example.invalid/v1",
                provider_name="tuzi-web",
            ),
            gemini=None,
            anthropic=None,
        )

        req = GenerateRequest(
            model="tuzi-web:chirp-v3-5",
            input=[Message(role="user", content=[Part.from_text("lofi hiphop beat")])],
            output=OutputSpec(
                modalities=["audio"], audio=OutputAudioSpec(voice="alloy", format="mp3")
            ),
            wait=True,
        )

        calls: list[dict[str, object]] = []

        def fake_request_json(*, method, url, headers=None, json_body=None, **_kwargs):
            calls.append({"method": method, "url": url, "json_body": json_body})
            if url == "https://example.invalid/suno/submit/music":
                return {
                    "clips": [
                        {"id": "clip_123", "audio_url": "https://example.invalid/a.mp3"}
                    ]
                }
            raise AssertionError(f"unexpected request_json call: {method} {url}")

        with patch("gravtice.genai.providers.tuzi.request_json") as request_json:
            request_json.side_effect = fake_request_json
            resp = tuzi.generate(req, stream=False)

        self.assertEqual(resp.status, "completed")
        parts = [p for m in resp.output for p in m.content]
        self.assertEqual(len(parts), 1)
        self.assertEqual(parts[0].type, "audio")
        self.assertIsNotNone(parts[0].source)
        self.assertEqual(getattr(parts[0].source, "kind", None), "url")
        self.assertEqual(
            getattr(parts[0].source, "url", None), "https://example.invalid/a.mp3"
        )
        self.assertEqual(len(calls), 1)

        submit = next(
            (
                c
                for c in calls
                if c["url"] == "https://example.invalid/suno/submit/music"
            ),
            None,
        )
        self.assertIsNotNone(submit)
        body = submit["json_body"]
        self.assertIsInstance(body, dict)
        self.assertEqual(body.get("mv"), "chirp-v3-5")

    def test_tuzi_web_chirp_v4_sets_mv(self) -> None:
        from gravtice.genai.providers import TuziAdapter
        from gravtice.genai.providers.openai import OpenAIAdapter
        from gravtice.genai.types import (
            GenerateRequest,
            Message,
            OutputAudioSpec,
            OutputSpec,
            Part,
        )

        tuzi = TuziAdapter(
            openai=OpenAIAdapter(
                api_key="__demo__",
                base_url="https://example.invalid/v1",
                provider_name="tuzi-web",
            ),
            gemini=None,
            anthropic=None,
        )

        req = GenerateRequest(
            model="tuzi-web:chirp-v4",
            input=[Message(role="user", content=[Part.from_text("lofi hiphop beat")])],
            output=OutputSpec(
                modalities=["audio"], audio=OutputAudioSpec(voice="alloy", format="mp3")
            ),
            wait=True,
        )

        calls: list[dict[str, object]] = []

        def fake_request_json(*, method, url, headers=None, json_body=None, **_kwargs):
            calls.append({"method": method, "url": url, "json_body": json_body})
            if url == "https://example.invalid/suno/submit/music":
                return {
                    "clips": [
                        {"id": "clip_123", "audio_url": "https://example.invalid/a.mp3"}
                    ]
                }
            raise AssertionError(f"unexpected request_json call: {method} {url}")

        with patch("gravtice.genai.providers.tuzi.request_json") as request_json:
            request_json.side_effect = fake_request_json
            resp = tuzi.generate(req, stream=False)

        self.assertEqual(resp.status, "completed")
        parts = [p for m in resp.output for p in m.content]
        self.assertEqual(len(parts), 1)
        self.assertEqual(parts[0].type, "audio")
        self.assertEqual(
            getattr(parts[0].source, "url", None), "https://example.invalid/a.mp3"
        )

        self.assertEqual(len(calls), 1)
        body = calls[0]["json_body"]
        self.assertIsInstance(body, dict)
        self.assertEqual(body.get("mv"), "chirp-v4")

    def test_tuzi_web_chirp_v3_5_waits_via_feed(self) -> None:
        from gravtice.genai.providers import TuziAdapter
        from gravtice.genai.providers.openai import OpenAIAdapter
        from gravtice.genai.types import (
            GenerateRequest,
            Message,
            OutputAudioSpec,
            OutputSpec,
            Part,
        )

        tuzi = TuziAdapter(
            openai=OpenAIAdapter(
                api_key="__demo__",
                base_url="https://example.invalid/v1",
                provider_name="tuzi-web",
            ),
            gemini=None,
            anthropic=None,
        )

        req = GenerateRequest(
            model="tuzi-web:chirp-v3-5",
            input=[Message(role="user", content=[Part.from_text("lofi hiphop beat")])],
            output=OutputSpec(
                modalities=["audio"], audio=OutputAudioSpec(voice="alloy", format="mp3")
            ),
            wait=True,
        )

        calls: list[str] = []

        def fake_request_json(*, method, url, headers=None, json_body=None, **_kwargs):
            calls.append(url)
            if url == "https://example.invalid/suno/submit/music":
                return {"clips": [{"id": "clip_123"}]}
            if url == "https://example.invalid/suno/feed?ids=clip_123":
                return {
                    "clips": [
                        {
                            "id": "clip_123",
                            "status": "complete",
                            "audio_url": "https://example.invalid/a.mp3",
                        }
                    ]
                }
            raise AssertionError(f"unexpected request_json call: {method} {url}")

        with patch("gravtice.genai.providers.tuzi.request_json") as request_json:
            request_json.side_effect = fake_request_json
            resp = tuzi.generate(req, stream=False)

        self.assertEqual(resp.status, "completed")
        parts = [p for m in resp.output for p in m.content]
        self.assertEqual(
            getattr(parts[0].source, "url", None), "https://example.invalid/a.mp3"
        )
        self.assertEqual(
            calls,
            [
                "https://example.invalid/suno/submit/music",
                "https://example.invalid/suno/feed?ids=clip_123",
            ],
        )

    def test_tuzi_web_chirp_v3_5_submit_music_prefers_task_id_fetch(self) -> None:
        from gravtice.genai.providers import TuziAdapter
        from gravtice.genai.providers.openai import OpenAIAdapter
        from gravtice.genai.types import (
            GenerateRequest,
            Message,
            OutputAudioSpec,
            OutputSpec,
            Part,
        )

        tuzi = TuziAdapter(
            openai=OpenAIAdapter(
                api_key="__demo__",
                base_url="https://example.invalid/v1",
                provider_name="tuzi-web",
            ),
            gemini=None,
            anthropic=None,
        )

        req = GenerateRequest(
            model="tuzi-web:chirp-v3-5",
            input=[Message(role="user", content=[Part.from_text("lofi hiphop beat")])],
            output=OutputSpec(
                modalities=["audio"], audio=OutputAudioSpec(voice="alloy", format="mp3")
            ),
            wait=True,
        )

        calls: list[str] = []

        def fake_request_json(*, method, url, headers=None, json_body=None, **_kwargs):
            calls.append(url)
            if url == "https://example.invalid/suno/submit/music":
                return {
                    "id": "task_123",
                    "clips": [{"id": "clip_123", "audio_url": ""}],
                }
            if url == "https://example.invalid/suno/fetch/task_123":
                return {
                    "data": {
                        "status": "SUCCESS",
                        "data": {"audio_url": "https://example.invalid/a.mp3"},
                    }
                }
            raise AssertionError(f"unexpected request_json call: {method} {url}")

        with patch("gravtice.genai.providers.tuzi.request_json") as request_json:
            request_json.side_effect = fake_request_json
            resp = tuzi.generate(req, stream=False)

        self.assertEqual(resp.status, "completed")
        parts = [p for m in resp.output for p in m.content]
        self.assertEqual(
            getattr(parts[0].source, "url", None), "https://example.invalid/a.mp3"
        )
        self.assertEqual(
            calls,
            [
                "https://example.invalid/suno/submit/music",
                "https://example.invalid/suno/fetch/task_123",
            ],
        )

    def test_tuzi_web_suno_submit_music_falls_back_to_fetch(self) -> None:
        from gravtice.genai.providers import TuziAdapter
        from gravtice.genai.providers.openai import OpenAIAdapter
        from gravtice.genai.types import (
            GenerateRequest,
            Message,
            OutputAudioSpec,
            OutputSpec,
            Part,
        )

        tuzi = TuziAdapter(
            openai=OpenAIAdapter(
                api_key="__demo__",
                base_url="https://example.invalid/v1",
                provider_name="tuzi-web",
            ),
            gemini=None,
            anthropic=None,
        )

        req = GenerateRequest(
            model="tuzi-web:chirp-v4",
            input=[Message(role="user", content=[Part.from_text("lofi hiphop beat")])],
            output=OutputSpec(
                modalities=["audio"], audio=OutputAudioSpec(voice="alloy", format="mp3")
            ),
            wait=True,
        )

        calls: list[str] = []

        def fake_request_json(*, method, url, headers=None, json_body=None, **_kwargs):
            calls.append(url)
            if url == "https://example.invalid/suno/submit/music":
                return {"id": "task_123"}
            if url == "https://example.invalid/suno/fetch/task_123":
                return {
                    "data": {
                        "status": "SUCCESS",
                        "data": {"audio_url": "https://example.invalid/a.mp3"},
                    }
                }
            raise AssertionError(f"unexpected request_json call: {method} {url}")

        with patch("gravtice.genai.providers.tuzi.request_json") as request_json:
            request_json.side_effect = fake_request_json
            resp = tuzi.generate(req, stream=False)

        self.assertEqual(resp.status, "completed")
        parts = [p for m in resp.output for p in m.content]
        self.assertEqual(
            getattr(parts[0].source, "url", None), "https://example.invalid/a.mp3"
        )
        self.assertEqual(
            calls,
            [
                "https://example.invalid/suno/submit/music",
                "https://example.invalid/suno/fetch/task_123",
            ],
        )

    def test_openai_images_supports_tuzi_wrapped_response(self) -> None:
        from gravtice.genai.providers.openai import OpenAIAdapter
        from gravtice.genai.types import (
            GenerateRequest,
            Message,
            OutputImageSpec,
            OutputSpec,
            Part,
        )

        adapter = OpenAIAdapter(
            api_key="__demo__",
            base_url="https://example.invalid/v1",
            provider_name="tuzi-web",
        )
        req = GenerateRequest(
            model="tuzi-web:chat-seededit",
            input=[Message(role="user", content=[Part.from_text("hi")])],
            output=OutputSpec(modalities=["image"], image=OutputImageSpec(n=1)),
            wait=True,
        )

        with patch("gravtice.genai.providers.openai.request_json") as request_json:
            request_json.return_value = {
                "data": {
                    "created": 1736160000,
                    "images": [{"url": "https://example.invalid/a.png"}],
                }
            }
            resp = adapter.generate(req, stream=False)

        self.assertEqual(resp.status, "completed")
        parts = [p for m in resp.output for p in m.content]
        self.assertEqual(len(parts), 1)
        self.assertEqual(parts[0].type, "image")
        self.assertIsNotNone(parts[0].source)
        self.assertEqual(getattr(parts[0].source, "kind", None), "url")

    def test_gemini_adapter_capabilities_support_imagen_and_native_audio(self) -> None:
        from gravtice.genai.providers.gemini import GeminiAdapter

        adapter = GeminiAdapter(
            api_key="__demo__",
            base_url="https://example.invalid",
            provider_name="google",
        )

        self.assertEqual(
            adapter.capabilities("imagen-4.0-generate-001").output_modalities, {"image"}
        )
        self.assertIn(
            "audio",
            adapter.capabilities(
                "gemini-2.5-flash-native-audio-latest"
            ).output_modalities,
        )

    def test_tuzi_openai_allows_non_sora_video_model_ids(self) -> None:
        from gravtice.genai.types import (
            GenerateRequest,
            Message,
            OutputSpec,
            OutputVideoSpec,
            Part,
        )
        from gravtice.genai.providers.openai import OpenAIAdapter

        req = GenerateRequest(
            model="tuzi-openai:pika-1.5",
            input=[Message(role="user", content=[Part.from_text("test")])],
            output=OutputSpec(
                modalities=["video"],
                video=OutputVideoSpec(duration_sec=4, aspect_ratio="16:9"),
            ),
            wait=False,
        )

        with patch(
            "gravtice.genai.providers.openai.request_streaming_body_json"
        ) as request_streaming_body_json:
            request_streaming_body_json.return_value = {"id": "vid_123"}
            adapter = OpenAIAdapter(
                api_key="__demo__",
                base_url="https://example.invalid/v1",
                provider_name="tuzi-openai",
            )
            out = adapter.generate(req, stream=False)
            self.assertEqual(out.status, "running")

    def test_tuzi_deepsearch_capabilities(self) -> None:
        from gravtice.genai.providers import TuziAdapter

        openai = object()
        tuzi = TuziAdapter(openai=openai, gemini=None, anthropic=None)

        for model in (
            "gemini-2.5-pro-deepsearch",
            "gemini-2.5-pro-deepsearch-async",
            "gemini-2.5-flash-deepsearch",
            "gemini-3-pro-deepsearch",
        ):
            with self.subTest(model=model):
                cap = tuzi.capabilities(model)
                self.assertEqual(cap.input_modalities, {"text"})
                self.assertEqual(cap.output_modalities, {"text"})
                self.assertFalse(cap.supports_stream)
                self.assertTrue(cap.supports_job)

    def test_tuzi_deepsearch_rejects_stream(self) -> None:
        from gravtice.genai import GenAIError
        from gravtice.genai.providers import TuziAdapter
        from gravtice.genai.providers.openai import OpenAIAdapter
        from gravtice.genai.types import GenerateRequest, Message, OutputSpec, Part

        openai_adapter = OpenAIAdapter(
            api_key="__demo__",
            base_url="https://example.invalid/v1",
            provider_name="tuzi-web",
        )
        tuzi = TuziAdapter(openai=openai_adapter, gemini=None, anthropic=None)

        req = GenerateRequest(
            model="tuzi-web:gemini-2.5-pro-deepsearch",
            input=[Message(role="user", content=[Part.from_text("test")])],
            output=OutputSpec(modalities=["text"]),
        )

        with self.assertRaises(GenAIError) as cm:
            tuzi.generate(req, stream=True)
        self.assertEqual(cm.exception.info.type, "InvalidRequestError")
        self.assertIn("streaming", str(cm.exception))

    def test_tuzi_deepsearch_submit_async_task(self) -> None:
        from gravtice.genai.providers import TuziAdapter
        from gravtice.genai.providers.openai import OpenAIAdapter
        from gravtice.genai.types import GenerateRequest, Message, OutputSpec, Part

        openai_adapter = OpenAIAdapter(
            api_key="test_key",
            base_url="https://api.tu-zi.com/v1",
            provider_name="tuzi-web",
        )
        tuzi = TuziAdapter(openai=openai_adapter, gemini=None, anthropic=None)

        req = GenerateRequest(
            model="tuzi-web:gemini-2.5-pro-deepsearch",
            input=[
                Message(role="user", content=[Part.from_text("analyze market trends")])
            ],
            output=OutputSpec(modalities=["text"]),
            wait=False,
        )

        with patch("gravtice.genai.providers.tuzi.request_json") as mock_request:
            mock_request.return_value = {
                "id": "task_abc123",
                "preview_url": "https://asyncdata.net/web/task_abc123",
                "source_url": "https://asyncdata.net/source/task_abc123",
            }
            out = tuzi.generate(req, stream=False)

            self.assertEqual(out.status, "running")
            self.assertIsNotNone(out.job)
            self.assertEqual(out.job.job_id, "task_abc123")
            self.assertEqual(out.job.poll_after_ms, 2_000)

            # Verify request was made to asyncdata.net with -async suffix
            call_args = mock_request.call_args
            self.assertIn("asyncdata.net/tran/", call_args.kwargs["url"])
            self.assertEqual(
                call_args.kwargs["json_body"]["model"],
                "gemini-2.5-pro-deepsearch-async",
            )

    def test_tuzi_suno_wait_fetch_audio_supports_list_data(self) -> None:
        from gravtice.genai.providers import TuziAdapter
        from gravtice.genai.providers.openai import OpenAIAdapter

        openai_adapter = OpenAIAdapter(
            api_key="test_key",
            base_url="https://api.tu-zi.com/v1",
            provider_name="tuzi-web",
        )
        tuzi = TuziAdapter(openai=openai_adapter, gemini=None, anthropic=None)

        with patch(
            "gravtice.genai.providers.tuzi.TuziAdapter._suno_fetch"
        ) as suno_fetch:
            suno_fetch.return_value = {
                "status": "SUCCESS",
                "data": [
                    {"audio_url": "https://cdn1.suno.ai/abc.mp3"},
                    {"audio_url": "https://cdn1.suno.ai/def.mp3"},
                ],
            }
            resp = tuzi._suno_wait_fetch_audio(
                task_id="task_123", model_id="chirp-v4", timeout_ms=10_000, wait=True
            )

        self.assertEqual(resp.status, "completed")
        parts = [p for m in resp.output for p in m.content]
        self.assertEqual(len(parts), 1)
        self.assertEqual(parts[0].type, "audio")
        self.assertIsNotNone(parts[0].source)
        self.assertEqual(getattr(parts[0].source, "kind", None), "url")
        self.assertEqual(
            getattr(parts[0].source, "url", None), "https://cdn1.suno.ai/abc.mp3"
        )

    def test_tuzi_suno_wait_fetch_audio_timeout_exposes_last_status(self) -> None:
        from gravtice.genai.providers import TuziAdapter
        from gravtice.genai.providers.openai import OpenAIAdapter

        openai_adapter = OpenAIAdapter(
            api_key="test_key",
            base_url="https://api.tu-zi.com/v1",
            provider_name="tuzi-web",
        )
        tuzi = TuziAdapter(openai=openai_adapter, gemini=None, anthropic=None)

        class FakeTime:
            def __init__(self) -> None:
                self.t = 0.0

            def time(self) -> float:
                self.t += 0.5
                return self.t

            def sleep(self, seconds: float) -> None:
                self.t += float(seconds)

        fake = FakeTime()

        with (
            patch("gravtice.genai.providers.tuzi.time.time", side_effect=fake.time),
            patch("gravtice.genai.providers.tuzi.time.sleep", side_effect=fake.sleep),
            patch(
                "gravtice.genai.providers.tuzi.TuziAdapter._suno_fetch"
            ) as suno_fetch,
        ):
            suno_fetch.return_value = {"status": "PENDING", "data": {}}
            resp = tuzi._suno_wait_fetch_audio(
                task_id="task_123", model_id="chirp-v3-5", timeout_ms=1_000, wait=True
            )

        self.assertEqual(resp.status, "running")
        self.assertIsNotNone(resp.job)
        self.assertEqual(resp.job.job_id, "task_123")
        self.assertEqual(resp.job.last_status, "PENDING")
        self.assertIsNone(resp.job.last_detail)

    def test_tuzi_suno_wait_fetch_audio_accepts_complete_status(self) -> None:
        from gravtice.genai.providers import TuziAdapter
        from gravtice.genai.providers.openai import OpenAIAdapter

        openai_adapter = OpenAIAdapter(
            api_key="test_key",
            base_url="https://api.tu-zi.com/v1",
            provider_name="tuzi-web",
        )
        tuzi = TuziAdapter(openai=openai_adapter, gemini=None, anthropic=None)

        with patch(
            "gravtice.genai.providers.tuzi.TuziAdapter._suno_fetch"
        ) as suno_fetch:
            suno_fetch.return_value = {
                "status": "complete",
                "data": {"audio_url": "https://cdn1.suno.ai/abc.mp3"},
            }
            resp = tuzi._suno_wait_fetch_audio(
                task_id="task_123", model_id="chirp-v4", timeout_ms=10_000, wait=True
            )

        self.assertEqual(resp.status, "completed")
        parts = [p for m in resp.output for p in m.content]
        self.assertEqual(len(parts), 1)
        self.assertEqual(parts[0].type, "audio")
        self.assertEqual(
            getattr(parts[0].source, "url", None), "https://cdn1.suno.ai/abc.mp3"
        )



from __future__ import annotations

MODELS: list[str] = [
    "claude-haiku-4-5-20251001",
    "claude-haiku-4-5-20251001-thinking",
    "claude-sonnet-4-5-20250929",
    "claude-sonnet-4-5-20250929-thinking",
    "claude-opus-4-5-20251101",
    "claude-opus-4-5-20251101-thinking",
    "deepseek-chat",
    "deepseek-coder",
    "deepseek-r1",
    "deepseek-r1-all",
    "deepseek-r1-search",
    "deepseek-r1-search-pro",
    "deepseek-r1-search-pro-thinking",
    "deepseek-search",
    "deepseek-v3",
    "deepseek-v3.2",
    "deepseek-v3.2-exp-thinking",
    "deepseek-v3.2-thinking",
    "doubao-seed-1-8-251228",
    "doubao-seed-1-8-251228-thinking",
    "gemini-2.5-flash",
    "gemini-2.5-flash-deepsearch",
    "gemini-2.5-pro",
    "gemini-2.5-pro-deepsearch",
    "gemini-3-flash-preview",
    "gemini-3-flash-preview-all",
    "gemini-3-flash-preview-thinking",
    "gemini-3-pro",
    "gemini-3-pro-deepsearch",
    "gemini-3-pro-preview",
    "gemini-3-pro-preview-thinking",
    "gemini-3-pro-thinking",
    "glm-4.5",
    "glm-4.5-air",
    "glm-4.5-airx",
    "glm-4.5-flash",
    "glm-4.5-x",
    "glm-4v",
    "gpt-4o",
    "gpt-4o-mini",
    "gpt-4o-search",
    "gpt-4o-study",
    "gpt-5",
    "gpt-5-mini",
    "gpt-5-nano",
    "gpt-5-pro",
    "gpt-5-thinking-all",
    "gpt-5.1",
    "gpt-5.1-thinking",
    "gpt-5.1-thinking-all",
    "gpt-5.2",
    "gpt-oss-120b",
    "gpt-oss-20b",
    "grok-3",
    "grok-3-deepersearch",
    "grok-3-deepsearch",
    "grok-3-reasoner",
    "grok-3-think",
    "grok-4",
    "grok-4.1",
    "kimi-k2-instruct",
    "o1",
    "o1-mini",
    "o1-pro",
    "o1-pro-all",
    "o3",
    "o3-all",
    "o3-mini",
    "o3-mini-high",
    "o3-pro",
    "o4-mini",
    "qwen2.5-72b-instruct",
    "qwen3-235b-a22b",
    "qwen3-32b",
    "search",
    "search-gpts",
    "search-gpts-chat",
    "dall-e-3",
    "doubao-seedream-4-0-250828",
    "doubao-seedream-4-5-251128",
    "flux-kontext-max",
    "flux-kontext-pro",
    "gemini-2.5-flash-image",
    "gemini-3-pro-image-preview",
    "gemini-3-pro-image-preview-2k",
    "gemini-3-pro-image-preview-4k",
    "gpt-image-1",
    "gpt-image-1.5",
    "kling_image",
    "seededit",
    "seedream-3.0",
    "seedream-4-0-250828",
    "seedream-v4",
    "kling-video-o1",
    "kling-video-o1-edit",
    "kling_video",
    "sora-16:9-480p-10s",
    "sora-16:9-480p-5s",
    "sora-16:9-720p-5s",
    "sora-1:1-480p-10s",
    "sora-1:1-480p-5s",
    "sora-1:1-720p-5s",
    "sora-2",
    "sora-2-character",
    "sora-2-pro",
    "sora-2-pro-character",
    "sora-9:16-480p-10",
    "sora-9:16-480p-5s",
    "sora-9:16-720p-5s",
    "veo2",
    "veo2-fast",
    "veo2-fast-frames",
    "veo2-pro",
    "veo3",
    "veo3-fast",
    "veo3-fast-frames",
    "veo3-frames",
    "veo3-pro",
    "veo3-pro-frames",
    "veo3.1",
    "veo3.1-components",
    "veo3.1-pro",
    "chirp-auk",
    "chirp-bluejay",
    "chirp-crow",
    "chirp-v3-0",
    "chirp-v3-5",
    "chirp-v4",
    "sonar-medium-chat",
    "sonar-medium-online",
    "sonar-small-chat",
    "sonar-small-online",
]



import asyncio
import logging
import os
import unittest
from unittest.mock import patch
from urllib.parse import urlparse


class TestMcpArtifactEviction(unittest.TestCase):
    def test_max_artifacts_evicts_oldest(self) -> None:
        try:
            from mcp.server.fastmcp import FastMCP  # noqa: F401
        except ModuleNotFoundError:
            self.skipTest("missing dependency: mcp")

        try:
            from starlette.testclient import TestClient
        except ModuleNotFoundError:
            self.skipTest("missing dependency: starlette")

        from gravtice.genai.mcp_server import build_server
        from gravtice.genai.types import (
            Capability,
            GenerateResponse,
            Message,
            Part,
            PartSourceBytes,
        )

        logging.getLogger("httpx").setLevel(logging.WARNING)
        logging.getLogger("httpcore").setLevel(logging.WARNING)

        resp = GenerateResponse(
            id="r1",
            provider="openai",
            model="openai:gpt-image-1",
            status="completed",
            output=[
                Message(
                    role="assistant",
                    content=[
                        Part(
                            type="image",
                            mime_type="image/png",
                            source=PartSourceBytes(data="MTIzNA==", encoding="base64"),
                        )
                    ],
                )
            ],
        )
        req_dict = {
            "model": "openai:gpt-image-1",
            "input": [{"role": "user", "content": [{"type": "text", "text": "x"}]}],
            "output": {"modalities": ["image"]},
        }

        with patch.dict(
            os.environ,
            {
                "GENAI_CALLING_MAX_INLINE_BASE64_CHARS": "1",
                "GENAI_CALLING_MAX_ARTIFACTS": "1",
                "GENAI_CALLING_MAX_ARTIFACT_BYTES": "1024",
            },
            clear=False,
        ):
            adapter = type(
                "DummyAdapter",
                (),
                {
                    "capabilities": lambda self, _: Capability(
                        input_modalities={"text"},
                        output_modalities={"image"},
                        supports_stream=False,
                        supports_job=False,
                    ),
                    "generate": lambda self, *_args, **_kwargs: resp,
                },
            )()
            with patch("gravtice.genai.client.Client._adapter", return_value=adapter):
                server = build_server(host="127.0.0.1", port=7001)

                out1 = server.call_tool("generate", {"request": req_dict})
                if asyncio.iscoroutine(out1):
                    out1 = asyncio.run(out1)
                if (
                    isinstance(out1, tuple)
                    and len(out1) == 2
                    and isinstance(out1[1], dict)
                ):
                    out1 = out1[1]

                out2 = server.call_tool("generate", {"request": req_dict})
                if asyncio.iscoroutine(out2):
                    out2 = asyncio.run(out2)
                if (
                    isinstance(out2, tuple)
                    and len(out2) == 2
                    and isinstance(out2[1], dict)
                ):
                    out2 = out2[1]

                self.assertIsInstance(out1, dict)
                self.assertIsInstance(out2, dict)

                url1 = out1["output"][0]["content"][0]["source"]["url"]
                url2 = out2["output"][0]["content"][0]["source"]["url"]
                artifact_id1 = url1.rsplit("/", 1)[-1]
                artifact_id2 = url2.rsplit("/", 1)[-1]
                self.assertNotEqual(artifact_id1, artifact_id2)

                app = server.streamable_http_app()
                client = TestClient(app)
                r2 = client.get(f"/artifact/{artifact_id2}")
                self.assertEqual(r2.status_code, 200)
                r1 = client.get(f"/artifact/{artifact_id1}")
                self.assertEqual(r1.status_code, 404)

    def test_bearer_signed_artifact_url_allows_headerless_download(self) -> None:
        try:
            from mcp.server.fastmcp import FastMCP  # noqa: F401
        except ModuleNotFoundError:
            self.skipTest("missing dependency: mcp")

        try:
            from starlette.testclient import TestClient
        except ModuleNotFoundError:
            self.skipTest("missing dependency: starlette")

        from gravtice.genai.mcp_server import _BearerAuthMiddleware, build_server
        from gravtice.genai.types import (
            Capability,
            GenerateResponse,
            Message,
            Part,
            PartSourceBytes,
        )

        logging.getLogger("httpx").setLevel(logging.WARNING)
        logging.getLogger("httpcore").setLevel(logging.WARNING)

        bearer = "test-token"
        resp = GenerateResponse(
            id="r1",
            provider="openai",
            model="openai:gpt-image-1",
            status="completed",
            output=[
                Message(
                    role="assistant",
                    content=[
                        Part(
                            type="image",
                            mime_type="image/png",
                            source=PartSourceBytes(data="MTIzNA==", encoding="base64"),
                        )
                    ],
                )
            ],
        )
        req_dict = {
            "model": "openai:gpt-image-1",
            "input": [{"role": "user", "content": [{"type": "text", "text": "x"}]}],
            "output": {"modalities": ["image"]},
        }

        with patch.dict(
            os.environ,
            {
                "GENAI_CALLING_MAX_INLINE_BASE64_CHARS": "1",
                "GENAI_CALLING_MAX_ARTIFACTS": "4",
                "GENAI_CALLING_MAX_ARTIFACT_BYTES": "1024",
                "GENAI_CALLING_ARTIFACT_URL_TTL_SECONDS": "600",
            },
            clear=False,
        ):
            adapter = type(
                "DummyAdapter",
                (),
                {
                    "capabilities": lambda self, _: Capability(
                        input_modalities={"text"},
                        output_modalities={"image"},
                        supports_stream=False,
                        supports_job=False,
                    ),
                    "generate": lambda self, *_args, **_kwargs: resp,
                },
            )()
            with patch("gravtice.genai.client.Client._adapter", return_value=adapter):
                server = build_server(host="127.0.0.1", port=7001, bearer_token=bearer)
                out = server.call_tool("generate", {"request": req_dict})
                if asyncio.iscoroutine(out):
                    out = asyncio.run(out)
                if (
                    isinstance(out, tuple)
                    and len(out) == 2
                    and isinstance(out[1], dict)
                ):
                    out = out[1]

                url = out["output"][0]["content"][0]["source"]["url"]
                parsed = urlparse(url)
                self.assertEqual(parsed.path.split("/", 2)[1], "artifact")
                self.assertIn("sig=", parsed.query)
                self.assertIn("exp=", parsed.query)
                artifact_id = parsed.path.rsplit("/", 1)[-1]

                app = server.streamable_http_app()
                app.add_middleware(_BearerAuthMiddleware, token=bearer)
                client = TestClient(app)

                r_missing = client.get(f"/artifact/{artifact_id}")
                self.assertEqual(r_missing.status_code, 401)

                r_header = client.get(
                    f"/artifact/{artifact_id}",
                    headers={"Authorization": f"Bearer {bearer}"},
                )
                self.assertEqual(r_header.status_code, 200)

                r_signed = client.get(f"{parsed.path}?{parsed.query}")
                self.assertEqual(r_signed.status_code, 200)



from __future__ import annotations

MODELS: list[str] = [
    "aqa",
    "deep-research-pro-preview-12-2025",
    "gemini-2.0-flash",
    "gemini-2.0-flash-lite",
    "gemini-2.5-computer-use-preview-10-2025",
    "gemini-2.5-flash",
    "gemini-2.5-flash-lite",
    "gemini-2.5-flash-native-audio-latest",
    "gemini-2.5-pro",
    "gemini-3-flash-preview",
    "gemini-3-pro-preview",
    "gemini-exp-1206",
    "gemini-flash-latest",
    "gemini-flash-lite-latest",
    "gemini-pro-latest",
    "gemini-robotics-er-1.5-preview",
    "gemma-3-12b-it",
    "gemma-3-1b-it",
    "gemma-3-27b-it",
    "gemma-3-4b-it",
    "gemma-3n-e2b-it",
    "gemma-3n-e4b-it",
    "gemini-2.5-flash-image",
    "gemini-3-pro-image-preview",
    "imagen-4.0-fast-generate-001",
    "imagen-4.0-generate-001",
    "imagen-4.0-generate-preview-06-06",
    "imagen-4.0-ultra-generate-001",
    "veo-2.0-generate-001",
    "veo-3.0-fast-generate-001",
    "veo-3.0-generate-001",
    "veo-3.1-fast-generate-preview",
    "veo-3.1-generate-preview",
    "gemini-2.5-flash-preview-tts",
    "gemini-2.5-pro-preview-tts",
    "embedding-001",
    "embedding-gecko-001",
    "gemini-embedding-001",
    "gemini-embedding-exp",
    "gemini-embedding-exp-03-07",
    "text-embedding-004",
]



from __future__ import annotations

import copy
from typing import Any


_MAPPINGS: list[dict[str, Any]] = [
    # === OpenAI (chat_completions) ===
    {
        "provider": "openai",
        "protocol": "chat_completions",
        "operation": "common",
        "from": "params.idempotency_key",
        "to": "header.Idempotency-Key",
    },
    {
        "provider": "openai",
        "protocol": "chat_completions",
        "operation": "chat",
        "from": "params.temperature",
        "to": "body.temperature",
    },
    {
        "provider": "openai",
        "protocol": "chat_completions",
        "operation": "chat",
        "from": "params.top_p",
        "to": "body.top_p",
    },
    {
        "provider": "openai",
        "protocol": "chat_completions",
        "operation": "chat",
        "from": "params.seed",
        "to": "body.seed",
    },
    {
        "provider": "openai",
        "protocol": "chat_completions",
        "operation": "chat",
        "from": "params.reasoning.effort",
        "to": "body.reasoning_effort",
        "notes": "reasoning models（如 gpt-5/o 系列）",
    },
    {
        "provider": "openai",
        "protocol": "chat_completions",
        "operation": "chat",
        "from": "output.text.max_output_tokens | params.max_output_tokens",
        "to": "body.max_completion_tokens",
    },
    {
        "provider": "openai",
        "protocol": "chat_completions",
        "operation": "chat",
        "from": "params.stop",
        "to": "body.stop",
    },
    {
        "provider": "openai",
        "protocol": "chat_completions",
        "operation": "chat",
        "from": "provider_options['openai']",
        "to": "body.* (merge)",
        "notes": "显式透传；禁止覆盖 SDK 已设置字段",
    },
    {
        "provider": "openai",
        "protocol": "chat_completions",
        "operation": "chat",
        "from": "output.text.format/json_schema",
        "to": "body.response_format",
        "notes": "json_object/json_schema（text-only）",
    },
    {
        "provider": "openai",
        "protocol": "chat_completions",
        "operation": "chat",
        "from": "input[].text",
        "to": "messages[].content[].{type:text}",
    },
    {
        "provider": "openai",
        "protocol": "chat_completions",
        "operation": "chat",
        "from": "input[].image(bytes/path/url)",
        "to": "messages[].content[].{type:image_url}",
        "notes": "bytes/path 会内联为 data:...;base64",
    },
    {
        "provider": "openai",
        "protocol": "chat_completions",
        "operation": "chat",
        "from": "input[].audio(bytes/path/url)",
        "to": "messages[].content[].{type:input_audio}",
        "notes": "bytes/path/url 会转 base64；format 由 mime 推断(wav/mp3/m4a)",
    },
    {
        "provider": "openai",
        "protocol": "chat_completions",
        "operation": "chat",
        "from": "output.modalities includes audio + output.audio.voice/format",
        "to": "body.modalities/body.audio",
        "notes": "音频输出为 message.audio(data base64 + transcript)",
    },
    {
        "provider": "openai",
        "protocol": "chat_completions",
        "operation": "images",
        "from": "output.image.n",
        "to": "body.n",
    },
    {
        "provider": "openai",
        "protocol": "chat_completions",
        "operation": "images",
        "from": "output.image.size",
        "to": "body.size",
    },
    {
        "provider": "openai",
        "protocol": "chat_completions",
        "operation": "images",
        "from": "output.image.format",
        "to": "body.response_format",
        "notes": "url | b64_json（bytes/base64 会落到 b64_json）",
    },
    {
        "provider": "openai",
        "protocol": "chat_completions",
        "operation": "images",
        "from": "provider_options['openai']",
        "to": "body.* (merge)",
        "notes": "显式透传 OpenAI 私参（quality/style/background 等）",
    },
    {
        "provider": "openai",
        "protocol": "chat_completions",
        "operation": "embeddings",
        "from": "input[].text",
        "to": "body.input",
    },
    {
        "provider": "openai",
        "protocol": "chat_completions",
        "operation": "embeddings",
        "from": "output.embedding.dimensions",
        "to": "body.dimensions",
        "notes": "仅 text-embedding-3-*",
    },
    {
        "provider": "openai",
        "protocol": "chat_completions",
        "operation": "embeddings",
        "from": "provider_options['openai']",
        "to": "body.* (merge)",
    },
    {
        "provider": "openai",
        "protocol": "chat_completions",
        "operation": "tts",
        "from": "output.audio.voice",
        "to": "body.voice",
    },
    {
        "provider": "openai",
        "protocol": "chat_completions",
        "operation": "tts",
        "from": "output.audio.format",
        "to": "body.response_format",
    },
    {
        "provider": "openai",
        "protocol": "chat_completions",
        "operation": "tts",
        "from": "input[].text",
        "to": "body.input",
    },
    {
        "provider": "openai",
        "protocol": "chat_completions",
        "operation": "tts",
        "from": "provider_options['openai']",
        "to": "body.* (merge)",
    },
    {
        "provider": "openai",
        "protocol": "chat_completions",
        "operation": "transcription",
        "from": "input[].audio",
        "to": "multipart.file",
    },
    {
        "provider": "openai",
        "protocol": "chat_completions",
        "operation": "transcription",
        "from": "params.temperature",
        "to": "multipart.temperature",
    },
    {
        "provider": "openai",
        "protocol": "chat_completions",
        "operation": "transcription",
        "from": "audio.meta.language",
        "to": "multipart.language",
    },
    {
        "provider": "openai",
        "protocol": "chat_completions",
        "operation": "transcription",
        "from": "audio.meta.transcription_prompt | text(meta.transcription_prompt=true)",
        "to": "multipart.prompt",
        "notes": "默认不自动采集任意 text part（避免意外影响转写）",
    },
    {
        "provider": "openai",
        "protocol": "chat_completions",
        "operation": "transcription",
        "from": "provider_options['openai']",
        "to": "multipart.* (merge)",
    },
    {
        "provider": "openai",
        "protocol": "chat_completions",
        "operation": "video",
        "from": "output.video.duration_sec",
        "to": "body.seconds",
        "notes": "4/8/12 秒就近取值",
    },
    {
        "provider": "openai",
        "protocol": "chat_completions",
        "operation": "video",
        "from": "output.video.aspect_ratio",
        "to": "body.size",
        "notes": "16:9/9:16 预置映射",
    },
    {
        "provider": "openai",
        "protocol": "chat_completions",
        "operation": "video",
        "from": "provider_options['openai']",
        "to": "body.* (merge)",
    },
    # === OpenAI (responses) ===
    {
        "provider": "openai",
        "protocol": "responses",
        "operation": "common",
        "from": "params.idempotency_key",
        "to": "header.Idempotency-Key",
    },
    {
        "provider": "openai",
        "protocol": "responses",
        "operation": "chat",
        "from": "params.temperature",
        "to": "body.temperature",
    },
    {
        "provider": "openai",
        "protocol": "responses",
        "operation": "chat",
        "from": "params.top_p",
        "to": "body.top_p",
    },
    {
        "provider": "openai",
        "protocol": "responses",
        "operation": "chat",
        "from": "params.reasoning.effort",
        "to": "body.reasoning.effort",
        "notes": "reasoning models（如 gpt-5/o 系列）",
    },
    {
        "provider": "openai",
        "protocol": "responses",
        "operation": "chat",
        "from": "output.text.max_output_tokens | params.max_output_tokens",
        "to": "body.max_output_tokens",
    },
    {
        "provider": "openai",
        "protocol": "responses",
        "operation": "chat",
        "from": "output.text.format/json_schema",
        "to": "body.text.format",
        "notes": "json_object/json_schema（text-only）",
    },
    {
        "provider": "openai",
        "protocol": "responses",
        "operation": "chat",
        "from": "input[].text",
        "to": "body.input[].content[].{type:input_text}",
    },
    {
        "provider": "openai",
        "protocol": "responses",
        "operation": "chat",
        "from": "input[].image(bytes/path/url)",
        "to": "body.input[].content[].{type:input_image}",
        "notes": "bytes/path 会内联为 data:...;base64",
    },
    {
        "provider": "openai",
        "protocol": "responses",
        "operation": "chat",
        "from": "params.seed / params.stop",
        "to": "not mapped",
        "notes": "OpenAI /responses 当前会返回 unknown_parameter",
    },
    {
        "provider": "openai",
        "protocol": "responses",
        "operation": "chat",
        "from": "provider_options['openai']",
        "to": "body.* (merge)",
        "notes": "显式透传；禁止覆盖 SDK 已设置字段",
    },
    # === Google AI Studio (Gemini Developer API) ===
    {
        "provider": "google",
        "protocol": "gemini",
        "operation": "generate",
        "from": "params.temperature",
        "to": "generationConfig.temperature",
    },
    {
        "provider": "google",
        "protocol": "gemini",
        "operation": "generate",
        "from": "params.top_p",
        "to": "generationConfig.topP",
    },
    {
        "provider": "google",
        "protocol": "gemini",
        "operation": "generate",
        "from": "params.seed",
        "to": "generationConfig.seed",
    },
    {
        "provider": "google",
        "protocol": "gemini",
        "operation": "generate",
        "from": "params.reasoning.effort",
        "to": "generationConfig.thinkingConfig.thinkingLevel | thinkingBudget(fallback)",
        "notes": "Gemini 3+ 优先 thinkingLevel（3 Pro 仅 low/high；3 Flash 支持 minimal/medium）；Gemini 2.5 / Robotics-ER 走 thinkingBudget；其他型号忽略",
    },
    {
        "provider": "google",
        "protocol": "gemini",
        "operation": "generate",
        "from": "output.text.max_output_tokens | params.max_output_tokens",
        "to": "generationConfig.maxOutputTokens",
    },
    {
        "provider": "google",
        "protocol": "gemini",
        "operation": "generate",
        "from": "params.stop",
        "to": "generationConfig.stopSequences",
    },
    {
        "provider": "google",
        "protocol": "gemini",
        "operation": "generate",
        "from": "output.text.format/json_schema",
        "to": "generationConfig.responseMimeType/responseSchema",
        "notes": "application/json（text-only）",
    },
    {
        "provider": "google",
        "protocol": "gemini",
        "operation": "generate",
        "from": "output.modalities",
        "to": "generationConfig.responseModalities",
        "notes": "TEXT/IMAGE/AUDIO（不支持 VIDEO 输出）",
    },
    {
        "provider": "google",
        "protocol": "gemini",
        "operation": "generate",
        "from": "output.image.n",
        "to": "generationConfig.candidateCount",
    },
    {
        "provider": "google",
        "protocol": "gemini",
        "operation": "generate",
        "from": "output.image.size",
        "to": "generationConfig.imageConfig.imageSize",
    },
    {
        "provider": "google",
        "protocol": "gemini",
        "operation": "generate",
        "from": "output.audio.voice/language",
        "to": "generationConfig.speechConfig",
        "notes": "voiceName + languageCode",
    },
    {
        "provider": "google",
        "protocol": "gemini",
        "operation": "generate",
        "from": "input[].(image/audio/video) path/url/bytes/ref",
        "to": "inlineData | fileData(fileUri)",
        "notes": "大文件会走 Files API 上传并等待 ACTIVE",
    },
    {
        "provider": "google",
        "protocol": "gemini",
        "operation": "generate",
        "from": "provider_options['google'|'gemini']",
        "to": "body.* / generationConfig.* (merge)",
        "notes": "显式透传；禁止覆盖 SDK 已设置字段",
    },
    {
        "provider": "google",
        "protocol": "gemini",
        "operation": "embedding",
        "from": "output.embedding.dimensions",
        "to": "outputDimensionality",
    },
    # === Anthropic (messages) ===
    {
        "provider": "anthropic",
        "protocol": "messages",
        "operation": "chat",
        "from": "params.reasoning.effort",
        "to": "body.thinking",
        "notes": "effort 映射到 thinking.enabled.budget_tokens（none 不传 thinking）",
    },
    # === OpenAI-compatible providers ===
    {
        "provider": "volcengine",
        "protocol": "chat_completions",
        "operation": "chat/embeddings/images",
        "from": "same as openai/chat_completions (subset)",
        "to": "same as openai/chat_completions (subset)",
        "notes": "Base URL 默认 https://ark.cn-beijing.volces.com/api/v3；API key=VOLCENGINE_API_KEY",
    },
    {
        "provider": "aliyun",
        "protocol": "chat_completions",
        "operation": "chat/embeddings",
        "from": "same as openai/chat_completions (subset)",
        "to": "same as openai/chat_completions (subset)",
        "notes": "Base URL 默认 https://dashscope.aliyuncs.com/compatible-mode/v1；API key=ALIYUN_API_KEY",
    },
    {
        "provider": "tuzi-openai",
        "protocol": "chat_completions",
        "operation": "chat/images/audio/embeddings",
        "from": "same as openai/chat_completions (subset)",
        "to": "same as openai/chat_completions (subset)",
        "notes": "Base URL 默认 https://api.tu-zi.com/v1；API key=TUZI_OPENAI_API_KEY（tuzi-web 则为 TUZI_WEB_API_KEY）",
    },
]


def get_parameter_mappings() -> list[dict[str, Any]]:
    """
    Return a JSON-friendly table of the current unified-params -> provider-params mapping.
    """
    return copy.deepcopy(_MAPPINGS)



from __future__ import annotations

MODELS: list[str] = [
    "gpt-4.1",
    "gpt-4.1-mini",
    "gpt-4.1-nano",
    "gpt-4o",
    "gpt-4o-mini",
    "chatgpt-4o-latest",
    "gpt-5",
    "gpt-5-codex",
    "gpt-5-mini",
    "gpt-5-nano",
    "gpt-5-pro",
    "gpt-5.1",
    "gpt-5.1-codex",
    "gpt-5.1-codex-max",
    "gpt-5.1-codex-mini",
    "gpt-5.2",
    "gpt-5.2-pro",
    "o1",
    "o1-pro",
    "o3",
    "o3-mini",
    "o4-mini",
    "gpt-4o-mini-transcribe",
    "gpt-4o-transcribe",
    "gpt-4o-transcribe-diarize",
    "whisper-1",
    "dall-e-2",
    "dall-e-3",
    "gpt-image-1",
    "gpt-image-1-mini",
    "gpt-image-1.5",
    "chatgpt-image-latest",
    "sora-2",
    "sora-2-pro",
    "gpt-4o-mini-tts",
    "tts-1",
    "tts-1-hd",
    "text-embedding-3-large",
    "text-embedding-3-small",
    "text-embedding-ada-002",
]



from __future__ import annotations

MODELS: list[str] = [
    "claude-instant-1.2",
    "claude-3-5-haiku-latest",
    "claude-3-5-sonnet-latest",
    "claude-3-7-sonnet-latest",
    "claude-3-7-sonnet-latest-thinking",
    "claude-sonnet-4-0",
    "claude-sonnet-4-0-thinking",
    "claude-opus-4-0",
    "claude-opus-4-0-thinking",
    "claude-opus-4-1",
    "claude-haiku-4-5",
    "claude-sonnet-4-5",
    "claude-opus-4-5",
    "Cursor-c4",
    "Cursor-c4-thinking",
    "Cursor-co4",
    "Cursor-co4-thinking",
]



import os
import unittest
from pathlib import Path
from tempfile import TemporaryDirectory
from unittest.mock import patch


class TestConfigEnvLoading(unittest.TestCase):
    def test_load_env_files_loads_global_home_env_as_lowest_priority(self) -> None:
        from gravtice.genai._internal.config import load_env_files

        with TemporaryDirectory() as tmpdir:
            root = Path(tmpdir) / "project"
            home = Path(tmpdir) / "home"
            root.mkdir()
            (home / ".genai-calling").mkdir(parents=True)
            global_env = home / ".genai-calling" / ".env"
            global_env.write_text(
                "GENAI_CALLING_OPENAI_API_KEY=global-key\n", encoding="utf-8"
            )

            with patch("gravtice.genai._internal.config.Path.home", return_value=home):
                with patch.dict(os.environ, {}, clear=True):
                    loaded = load_env_files(root=root)
                    value = os.environ.get("GENAI_CALLING_OPENAI_API_KEY")

            self.assertEqual(value, "global-key")
            self.assertEqual(loaded, [global_env])

    def test_project_env_overrides_global_home_env(self) -> None:
        from gravtice.genai._internal.config import load_env_files

        with TemporaryDirectory() as tmpdir:
            root = Path(tmpdir) / "project"
            home = Path(tmpdir) / "home"
            root.mkdir()
            (home / ".genai-calling").mkdir(parents=True)
            (root / ".env.local").write_text(
                "GENAI_CALLING_OPENAI_API_KEY=local-key\n", encoding="utf-8"
            )
            global_env = home / ".genai-calling" / ".env"
            global_env.write_text(
                "GENAI_CALLING_OPENAI_API_KEY=global-key\n", encoding="utf-8"
            )

            with patch("gravtice.genai._internal.config.Path.home", return_value=home):
                with patch.dict(os.environ, {}, clear=True):
                    loaded = load_env_files(root=root)
                    value = os.environ.get("GENAI_CALLING_OPENAI_API_KEY")

            self.assertEqual(value, "local-key")
            self.assertEqual(loaded, [root / ".env.local", global_env])

    def test_process_env_overrides_project_and_global_env(self) -> None:
        from gravtice.genai._internal.config import load_env_files

        with TemporaryDirectory() as tmpdir:
            root = Path(tmpdir) / "project"
            home = Path(tmpdir) / "home"
            root.mkdir()
            (home / ".genai-calling").mkdir(parents=True)
            (root / ".env.local").write_text(
                "GENAI_CALLING_OPENAI_API_KEY=local-key\n", encoding="utf-8"
            )
            (home / ".genai-calling" / ".env").write_text(
                "GENAI_CALLING_OPENAI_API_KEY=global-key\n", encoding="utf-8"
            )

            with patch("gravtice.genai._internal.config.Path.home", return_value=home):
                with patch.dict(
                    os.environ,
                    {"GENAI_CALLING_OPENAI_API_KEY": "process-key"},
                    clear=True,
                ):
                    load_env_files(root=root)
                    value = os.environ.get("GENAI_CALLING_OPENAI_API_KEY")

            self.assertEqual(value, "process-key")



import unittest
from unittest.mock import patch


class TestReasoningMapping(unittest.TestCase):
    def test_openai_chat_maps_reasoning_effort(self) -> None:
        from gravtice.genai.types import (
            GenerateParams,
            GenerateRequest,
            Message,
            OutputSpec,
            Part,
            ReasoningSpec,
        )
        from gravtice.genai.providers.openai import OpenAIAdapter

        req = GenerateRequest(
            model="openai:gpt-4o-mini",
            input=[Message(role="user", content=[Part.from_text("test")])],
            output=OutputSpec(modalities=["text"]),
            params=GenerateParams(reasoning=ReasoningSpec(effort="HIGH")),
        )

        with patch("gravtice.genai.providers.openai.request_json") as request_json:
            request_json.return_value = {
                "id": "chatcmpl_123",
                "choices": [{"message": {"content": "ok"}}],
            }
            adapter = OpenAIAdapter(
                api_key="__demo__",
                base_url="https://example.invalid/v1",
                provider_name="openai",
            )
            out = adapter.generate(req, stream=False)
            self.assertEqual(out.status, "completed")

            _, kwargs = request_json.call_args
            body = kwargs["json_body"]
            self.assertEqual(body.get("reasoning_effort"), "high")

    def test_openai_responses_maps_reasoning_effort(self) -> None:
        from gravtice.genai.types import (
            GenerateParams,
            GenerateRequest,
            Message,
            OutputSpec,
            Part,
            ReasoningSpec,
        )
        from gravtice.genai.providers.openai import OpenAIAdapter

        req = GenerateRequest(
            model="openai:gpt-5.2",
            input=[Message(role="user", content=[Part.from_text("test")])],
            output=OutputSpec(modalities=["text"]),
            params=GenerateParams(reasoning=ReasoningSpec(effort="low")),
        )

        with patch("gravtice.genai.providers.openai.request_json") as request_json:
            request_json.return_value = {
                "id": "resp_123",
                "status": "completed",
                "output": [
                    {
                        "type": "message",
                        "content": [{"type": "output_text", "text": "ok"}],
                    }
                ],
            }
            adapter = OpenAIAdapter(
                api_key="__demo__",
                base_url="https://example.invalid/v1",
                provider_name="openai",
                chat_api="responses",
            )
            out = adapter.generate(req, stream=False)
            self.assertEqual(out.status, "completed")

            _, kwargs = request_json.call_args
            body = kwargs["json_body"]
            self.assertEqual(body.get("reasoning"), {"effort": "low"})

    def test_anthropic_maps_reasoning_effort_to_budget(self) -> None:
        from gravtice.genai.types import (
            GenerateParams,
            GenerateRequest,
            Message,
            OutputSpec,
            OutputTextSpec,
            Part,
            ReasoningSpec,
        )
        from gravtice.genai.providers.anthropic import AnthropicAdapter

        req = GenerateRequest(
            model="anthropic:claude-sonnet-4-5",
            input=[Message(role="user", content=[Part.from_text("test")])],
            output=OutputSpec(
                modalities=["text"], text=OutputTextSpec(max_output_tokens=2048)
            ),
            params=GenerateParams(reasoning=ReasoningSpec(effort="minimal")),
        )

        with patch("gravtice.genai.providers.anthropic.request_json") as request_json:
            request_json.return_value = {
                "id": "msg_123",
                "content": [{"type": "text", "text": "ok"}],
            }
            adapter = AnthropicAdapter(
                api_key="__demo__",
                base_url="https://example.invalid",
                provider_name="anthropic",
            )
            out = adapter.generate(req, stream=False)
            self.assertEqual(out.status, "completed")

            _, kwargs = request_json.call_args
            body = kwargs["json_body"]
            self.assertEqual(
                body.get("thinking"), {"type": "enabled", "budget_tokens": 1024}
            )

    def test_gemini_maps_reasoning_effort_to_thinking_level_for_gemini3(self) -> None:
        from gravtice.genai.types import (
            GenerateParams,
            GenerateRequest,
            Message,
            OutputSpec,
            Part,
            ReasoningSpec,
        )
        from gravtice.genai.providers.gemini import GeminiAdapter

        req = GenerateRequest(
            model="google:gemini-3-pro-preview",
            input=[Message(role="user", content=[Part.from_text("test")])],
            output=OutputSpec(modalities=["text"]),
            params=GenerateParams(reasoning=ReasoningSpec(effort="high")),
        )

        with patch("gravtice.genai.providers.gemini.request_json") as request_json:
            request_json.return_value = {
                "candidates": [{"content": {"parts": [{"text": "ok"}]}}]
            }
            adapter = GeminiAdapter(
                api_key="__demo__",
                base_url="https://example.invalid",
                provider_name="google",
            )
            out = adapter.generate(req, stream=False)
            self.assertEqual(out.status, "completed")

            _, kwargs = request_json.call_args
            body = kwargs["json_body"]
            gen_cfg = body.get("generationConfig") or {}
            thinking_cfg = gen_cfg.get("thinkingConfig") or {}
            self.assertEqual(thinking_cfg.get("thinkingLevel"), "high")

    def test_gemini_maps_reasoning_effort_to_thinking_budget_for_gemini2(self) -> None:
        from gravtice.genai.types import (
            GenerateParams,
            GenerateRequest,
            Message,
            OutputSpec,
            Part,
            ReasoningSpec,
        )
        from gravtice.genai.providers.gemini import GeminiAdapter

        req = GenerateRequest(
            model="google:gemini-2.5-flash",
            input=[Message(role="user", content=[Part.from_text("test")])],
            output=OutputSpec(modalities=["text"]),
            params=GenerateParams(reasoning=ReasoningSpec(effort="low")),
        )

        with patch("gravtice.genai.providers.gemini.request_json") as request_json:
            request_json.return_value = {
                "candidates": [{"content": {"parts": [{"text": "ok"}]}}]
            }
            adapter = GeminiAdapter(
                api_key="__demo__",
                base_url="https://example.invalid",
                provider_name="google",
            )
            out = adapter.generate(req, stream=False)
            self.assertEqual(out.status, "completed")

            _, kwargs = request_json.call_args
            body = kwargs["json_body"]
            gen_cfg = body.get("generationConfig") or {}
            thinking_cfg = gen_cfg.get("thinkingConfig") or {}
            self.assertEqual(thinking_cfg.get("thinkingBudget"), 1024)

    def test_gemini_ignores_reasoning_for_unsupported_models(self) -> None:
        from gravtice.genai.types import (
            GenerateParams,
            GenerateRequest,
            Message,
            OutputSpec,
            Part,
            ReasoningSpec,
        )
        from gravtice.genai.providers.gemini import GeminiAdapter

        req = GenerateRequest(
            model="google:gemini-1.5-flash",
            input=[Message(role="user", content=[Part.from_text("test")])],
            output=OutputSpec(modalities=["text"]),
            params=GenerateParams(reasoning=ReasoningSpec(effort="high")),
        )

        with patch("gravtice.genai.providers.gemini.request_json") as request_json:
            request_json.return_value = {
                "candidates": [{"content": {"parts": [{"text": "ok"}]}}]
            }
            adapter = GeminiAdapter(
                api_key="__demo__",
                base_url="https://example.invalid",
                provider_name="google",
            )
            out = adapter.generate(req, stream=False)
            self.assertEqual(out.status, "completed")

            _, kwargs = request_json.call_args
            body = kwargs["json_body"]
            gen_cfg = body.get("generationConfig") or {}
            self.assertNotIn("thinkingConfig", gen_cfg)



from .output_parser import (
    DEFAULT_OUTPUT_PARSER_TOOL_NAME,
    build_output_parser_tool,
    extract_output_from_response,
    parse_output,
)

__all__ = [
    "DEFAULT_OUTPUT_PARSER_TOOL_NAME",
    "build_output_parser_tool",
    "extract_output_from_response",
    "parse_output",
]



import asyncio
import unittest
from unittest.mock import patch


def _call_tool(server, name: str, args: dict) -> dict:
    out = server.call_tool(name, args)
    if asyncio.iscoroutine(out):
        out = asyncio.run(out)
    if isinstance(out, tuple) and len(out) == 2 and isinstance(out[1], dict):
        out = out[1]
    if not isinstance(out, dict):
        raise AssertionError(f"unexpected tool output type: {type(out)}")
    return out


class TestMcpTokenRules(unittest.TestCase):
    def test_parse_bracket_rules_supports_models_and_providers(self) -> None:
        from gravtice.genai.mcp_server import _parse_mcp_token_scopes

        scopes = _parse_mcp_token_scopes(
            "t1: [openai google openai:gpt-4o-mini]; t2: [openai:gpt-4o-mini]"
        )
        self.assertEqual(set(scopes.keys()), {"t1", "t2"})

        t1 = scopes["t1"]
        self.assertIn("openai", t1.providers)
        self.assertIn("google", t1.providers)
        self.assertIn("openai", t1.providers_all)
        self.assertIn("google", t1.providers_all)

        t2 = scopes["t2"]
        self.assertIn("openai", t2.providers)
        self.assertNotIn("google", t2.providers)
        self.assertNotIn("openai", t2.providers_all)
        self.assertEqual(set(t2.models.get("openai") or set()), {"gpt-4o-mini"})

    def test_token_scopes_filter_providers_models_and_generate(self) -> None:
        try:
            from mcp.server.fastmcp import FastMCP  # noqa: F401
        except ModuleNotFoundError:
            self.skipTest("missing dependency: mcp")

        from gravtice.genai.mcp_server import (
            _REQUEST_TOKEN,
            _parse_mcp_token_scopes,
            build_server,
        )
        from gravtice.genai.types import GenerateResponse, Message, Part
        from mcp.server.fastmcp.exceptions import ToolError

        scopes = _parse_mcp_token_scopes(
            "t1: [openai google]; t2: [openai:gpt-4o-mini]"
        )

        def fake_adapter(_self, provider: str):
            return object()

        def fake_list_available_models(_self, provider: str, *, timeout_ms=None):  # noqa: ARG001
            if provider == "openai":
                return ["gpt-4o-mini", "gpt-4.1"]
            if provider == "google":
                return ["gemini-2.0-flash"]
            return []

        def fake_supported_for_provider(provider: str):
            if provider == "openai":
                return [
                    {
                        "model_id": "gpt-4o-mini",
                        "model": "openai:gpt-4o-mini",
                        "modes": ["sync"],
                        "input_modalities": ["text"],
                        "output_modalities": ["text"],
                    },
                    {
                        "model_id": "gpt-4.1",
                        "model": "openai:gpt-4.1",
                        "modes": ["sync"],
                        "input_modalities": ["text"],
                        "output_modalities": ["text"],
                    },
                ]
            if provider == "google":
                return [
                    {
                        "model_id": "gemini-2.0-flash",
                        "model": "google:gemini-2.0-flash",
                        "modes": ["sync"],
                        "input_modalities": ["text"],
                        "output_modalities": ["text"],
                    }
                ]
            return []

        resp = GenerateResponse(
            id="r1",
            provider="openai",
            model="openai:gpt-4o-mini",
            status="completed",
            output=[Message(role="assistant", content=[Part(type="text", text="ok")])],
        )

        with patch(
            "gravtice.genai.reference.get_model_catalog",
            return_value={"openai": [], "google": []},
        ):
            with patch(
                "gravtice.genai.reference.get_supported_providers",
                return_value=["openai", "google"],
            ):
                with patch(
                    "gravtice.genai.reference.get_sdk_supported_models_for_provider",
                    side_effect=fake_supported_for_provider,
                ):
                    with patch(
                        "gravtice.genai.client.Client._adapter", new=fake_adapter
                    ):
                        with patch(
                            "gravtice.genai.client.Client.list_available_models",
                            autospec=True,
                            side_effect=fake_list_available_models,
                        ) as list_mock:
                            with patch(
                                "gravtice.genai.client.Client.generate",
                                return_value=resp,
                            ) as gen_mock:
                                server = build_server(
                                    host="127.0.0.1", port=7001, token_scopes=scopes
                                )

                                ctx = _REQUEST_TOKEN.set("t2")
                                try:
                                    out = _call_tool(server, "list_providers", {})
                                    self.assertEqual(out["supported"], ["openai"])
                                    self.assertEqual(out["configured"], ["openai"])

                                    models = _call_tool(
                                        server,
                                        "list_available_models",
                                        {"provider": "openai"},
                                    )
                                    self.assertEqual(
                                        [m["model"] for m in models["models"]],
                                        ["openai:gpt-4o-mini"],
                                    )

                                    with self.assertRaises(ToolError):
                                        _call_tool(
                                            server,
                                            "list_available_models",
                                            {"provider": "google"},
                                        )

                                    all_models = _call_tool(
                                        server, "list_all_available_models", {}
                                    )
                                    self.assertEqual(
                                        [m["model"] for m in all_models["models"]],
                                        ["openai:gpt-4o-mini"],
                                    )
                                    providers = [
                                        c.args[1] for c in list_mock.call_args_list
                                    ]
                                    self.assertEqual(set(providers), {"openai"})

                                    req = {
                                        "model": "openai:gpt-4o-mini",
                                        "input": [
                                            {
                                                "role": "user",
                                                "content": [
                                                    {"type": "text", "text": "x"}
                                                ],
                                            }
                                        ],
                                        "output": {"modalities": ["text"]},
                                    }
                                    gen_out = _call_tool(
                                        server, "generate", {"request": req}
                                    )
                                    self.assertEqual(gen_out["status"], "completed")
                                    self.assertTrue(gen_mock.called)

                                    req_bad = dict(req)
                                    req_bad["model"] = "openai:gpt-4.1"
                                    with self.assertRaises(ToolError):
                                        _call_tool(
                                            server, "generate", {"request": req_bad}
                                        )
                                finally:
                                    _REQUEST_TOKEN.reset(ctx)



from .aliyun import AliyunAdapter
from .anthropic import AnthropicAdapter
from .gemini import GeminiAdapter
from .openai import OpenAIAdapter
from .tuzi import TuziAdapter
from .volcengine import VolcengineAdapter

__all__ = [
    "AliyunAdapter",
    "AnthropicAdapter",
    "GeminiAdapter",
    "OpenAIAdapter",
    "TuziAdapter",
    "VolcengineAdapter",
]



import asyncio
import os
import tempfile
import unittest
from unittest.mock import patch


class TestAnthropicImageUrlDoesNotCrash(unittest.TestCase):
    def test_messages_body_passes_proxy_url_to_downloader(self) -> None:
        from gravtice.genai.providers.anthropic import AnthropicAdapter
        from gravtice.genai.types import (
            GenerateRequest,
            Message,
            OutputSpec,
            Part,
            PartSourceUrl,
        )

        with tempfile.NamedTemporaryFile(
            prefix="genaisdk-test-", suffix=".bin", delete=False
        ) as f:
            tmp_path = f.name
            f.write(b"123")

        def _fake_download_to_tempfile(
            *,
            url: str,
            timeout_ms: int | None,
            max_bytes: int | None,
            proxy_url: str | None,
        ):  # type: ignore[no-untyped-def]
            self.assertEqual(url, "http://example.invalid/x.png")
            self.assertEqual(proxy_url, "http://proxy.local")
            return tmp_path

        adapter = AnthropicAdapter(api_key="__test__", proxy_url="http://proxy.local")
        req = GenerateRequest(
            model="anthropic:claude-sonnet-4",
            input=[
                Message(
                    role="user",
                    content=[
                        Part(
                            type="image",
                            mime_type="image/png",
                            source=PartSourceUrl(url="http://example.invalid/x.png"),
                        )
                    ],
                )
            ],
            output=OutputSpec(modalities=["text"]),
        )
        try:
            with patch(
                "gravtice.genai.providers.anthropic.download_to_tempfile",
                side_effect=_fake_download_to_tempfile,
            ):
                body = adapter._messages_body(req, stream=False)  # type: ignore[attr-defined]
        finally:
            try:
                os.unlink(tmp_path)
            except OSError:
                pass
        self.assertIsInstance(body, dict)


class TestUrlDownloadPrivateHostDnsBlocked(unittest.TestCase):
    def test_download_to_file_blocks_private_dns(self) -> None:
        from gravtice.genai import GenAIError
        from gravtice.genai._internal.http import download_to_file

        def _fake_getaddrinfo(host: str, port: object, proto: int):  # type: ignore[no-untyped-def]
            self.assertEqual(host, "evil.example")
            return [
                (socket.AF_INET, socket.SOCK_STREAM, proto, "", ("10.0.0.1", 0)),
            ]

        with tempfile.TemporaryDirectory(prefix="genaisdk-test-") as d:
            out_path = os.path.join(d, "out.bin")
            with patch.dict(
                os.environ, {"GENAI_CALLING_ALLOW_PRIVATE_URLS": "0"}, clear=False
            ):
                with patch(
                    "gravtice.genai._internal.http.socket.getaddrinfo"
                ) as mock_getaddrinfo:
                    import socket

                    mock_getaddrinfo.side_effect = _fake_getaddrinfo
                    with self.assertRaises(GenAIError) as cm:
                        download_to_file(
                            url="http://evil.example/x", output_path=out_path
                        )
        self.assertEqual(cm.exception.info.type, "InvalidRequestError")
        self.assertIn("private/loopback", cm.exception.info.message)


class TestOpenAIProviderOptionsNoOverride(unittest.TestCase):
    def test_images_provider_options_cannot_override_prompt(self) -> None:
        from gravtice.genai import GenAIError
        from gravtice.genai.providers.openai import OpenAIAdapter
        from gravtice.genai.types import GenerateRequest, Message, OutputSpec, Part

        adapter = OpenAIAdapter(api_key="__test__")
        req = GenerateRequest(
            model="openai:gpt-image-1",
            input=[Message(role="user", content=[Part.from_text("hi")])],
            output=OutputSpec(modalities=["image"]),
            provider_options={"openai": {"prompt": "override"}},
        )
        with self.assertRaises(GenAIError) as cm:
            adapter.generate(req, stream=False)
        self.assertEqual(cm.exception.info.type, "InvalidRequestError")
        self.assertIn("cannot override body.prompt", cm.exception.info.message)

    def test_transcribe_provider_options_cannot_override_model(self) -> None:
        from gravtice.genai import GenAIError
        from gravtice.genai.providers.openai import OpenAIAdapter
        from gravtice.genai.types import (
            GenerateRequest,
            Message,
            OutputSpec,
            Part,
            PartSourceBytes,
        )

        adapter = OpenAIAdapter(api_key="__test__")
        req = GenerateRequest(
            model="openai:whisper-1",
            input=[
                Message(
                    role="user",
                    content=[
                        Part(
                            type="audio",
                            mime_type="audio/wav",
                            source=PartSourceBytes(data="AA==", encoding="base64"),
                        )
                    ],
                )
            ],
            output=OutputSpec(modalities=["text"]),
            provider_options={"openai": {"model": "override"}},
        )
        with self.assertRaises(GenAIError) as cm:
            adapter.generate(req, stream=False)
        self.assertEqual(cm.exception.info.type, "InvalidRequestError")
        self.assertIn("cannot override fields.model", cm.exception.info.message)


class TestClientInputModalitiesValidated(unittest.TestCase):
    def test_client_rejects_unsupported_input_modality(self) -> None:
        from gravtice.genai import GenAIError
        from gravtice.genai.client import Client
        from gravtice.genai.types import (
            Capability,
            GenerateRequest,
            Message,
            OutputSpec,
            Part,
            PartSourceBytes,
        )

        class _DummyAdapter:
            def capabilities(self, model_id: str) -> Capability:  # noqa: ARG002
                return Capability(
                    input_modalities={"text"},
                    output_modalities={"text"},
                    supports_stream=False,
                    supports_job=False,
                )

            def generate(self, request: object, *, stream: bool):  # noqa: ARG002
                raise AssertionError("adapter.generate should not be called")

        client = Client()
        req = GenerateRequest(
            model="dummy:demo",
            input=[
                Message(
                    role="user",
                    content=[
                        Part(
                            type="audio",
                            mime_type="audio/wav",
                            source=PartSourceBytes(data="AA==", encoding="base64"),
                        )
                    ],
                )
            ],
            output=OutputSpec(modalities=["text"]),
        )
        with patch.object(client, "_adapter", return_value=_DummyAdapter()):
            with self.assertRaises(GenAIError) as cm:
                client.generate(req)
        self.assertEqual(cm.exception.info.type, "NotSupportedError")
        self.assertIn(
            "requested input modalities not supported", cm.exception.info.message
        )


class TestPartValidation(unittest.TestCase):
    def test_image_requires_source(self) -> None:
        from gravtice.genai.types import Part

        with self.assertRaises(ValueError) as cm:
            Part(type="image", mime_type="image/png", source=None)
        self.assertIn("requires source", str(cm.exception))


class TestMcpArtifactByteLimit(unittest.TestCase):
    def test_generate_converts_base64_to_artifact_url_under_limit(self) -> None:
        try:
            from mcp.server.fastmcp import FastMCP  # noqa: F401
        except ModuleNotFoundError:
            self.skipTest("missing dependency: mcp")

        from gravtice.genai.mcp_server import build_server
        from gravtice.genai.types import (
            Capability,
            GenerateResponse,
            Message,
            Part,
            PartSourceBytes,
        )

        resp = GenerateResponse(
            id="r1",
            provider="openai",
            model="openai:gpt-image-1",
            status="completed",
            output=[
                Message(
                    role="assistant",
                    content=[
                        Part(
                            type="image",
                            mime_type="image/png",
                            source=PartSourceBytes(data="MTIzNA==", encoding="base64"),
                        )
                    ],
                )
            ],
        )
        req_dict = {
            "model": "openai:gpt-image-1",
            "input": [{"role": "user", "content": [{"type": "text", "text": "x"}]}],
            "output": {"modalities": ["image"]},
        }

        with patch.dict(
            os.environ,
            {
                "GENAI_CALLING_MAX_INLINE_BASE64_CHARS": "4",
                "GENAI_CALLING_MAX_ARTIFACTS": "64",
                "GENAI_CALLING_MAX_ARTIFACT_BYTES": "1024",
                "GENAI_CALLING_MCP_PUBLIC_BASE_URL": "",
            },
            clear=False,
        ):
            adapter = type(
                "DummyAdapter",
                (),
                {
                    "capabilities": lambda self, _: Capability(
                        input_modalities={"text"},
                        output_modalities={"image"},
                        supports_stream=False,
                        supports_job=False,
                    ),
                    "generate": lambda self, *_args, **_kwargs: resp,
                },
            )()
            with patch("gravtice.genai.client.Client._adapter", return_value=adapter):
                server = build_server(host="127.0.0.1", port=7001)
                out = server.call_tool("generate", {"request": req_dict})
                if asyncio.iscoroutine(out):
                    out = asyncio.run(out)

        if isinstance(out, tuple) and len(out) == 2 and isinstance(out[1], dict):
            out = out[1]
        self.assertIsInstance(out, dict)
        part = out["output"][0]["content"][0]
        self.assertEqual(part["source"]["kind"], "url")
        self.assertIn("http://127.0.0.1:7001/artifact/", part["source"]["url"])

    def test_generate_keeps_base64_when_over_artifact_bytes_limit(self) -> None:
        try:
            from mcp.server.fastmcp import FastMCP  # noqa: F401
        except ModuleNotFoundError:
            self.skipTest("missing dependency: mcp")

        from gravtice.genai.mcp_server import build_server
        from gravtice.genai.types import (
            Capability,
            GenerateResponse,
            Message,
            Part,
            PartSourceBytes,
        )

        resp = GenerateResponse(
            id="r1",
            provider="openai",
            model="openai:gpt-image-1",
            status="completed",
            output=[
                Message(
                    role="assistant",
                    content=[
                        Part(
                            type="image",
                            mime_type="image/png",
                            source=PartSourceBytes(data="MTIzNA==", encoding="base64"),
                        )
                    ],
                )
            ],
        )
        req_dict = {
            "model": "openai:gpt-image-1",
            "input": [{"role": "user", "content": [{"type": "text", "text": "x"}]}],
            "output": {"modalities": ["image"]},
        }

        with patch.dict(
            os.environ,
            {
                "GENAI_CALLING_MAX_INLINE_BASE64_CHARS": "4",
                "GENAI_CALLING_MAX_ARTIFACTS": "64",
                "GENAI_CALLING_MAX_ARTIFACT_BYTES": "3",
            },
            clear=False,
        ):
            adapter = type(
                "DummyAdapter",
                (),
                {
                    "capabilities": lambda self, _: Capability(
                        input_modalities={"text"},
                        output_modalities={"image"},
                        supports_stream=False,
                        supports_job=False,
                    ),
                    "generate": lambda self, *_args, **_kwargs: resp,
                },
            )()
            with patch("gravtice.genai.client.Client._adapter", return_value=adapter):
                server = build_server(host="127.0.0.1", port=7001)
                out = server.call_tool("generate", {"request": req_dict})
                if asyncio.iscoroutine(out):
                    out = asyncio.run(out)

        if isinstance(out, tuple) and len(out) == 2 and isinstance(out[1], dict):
            out = out[1]
        self.assertIsInstance(out, dict)
        part = out["output"][0]["content"][0]
        self.assertEqual(part["source"]["kind"], "bytes")
        self.assertEqual(part["source"]["encoding"], "base64")
